function [fval info B Q P] = gausscosh(A,u,theta,opts,show)
%GAUSSCOSH Compute u'*cosh(sqrt(AA'))*u via Gauss quadrature.
%   fval = GAUSSCOSH(A,u,theta) computes u'*cosh(sqrt(AA'))*u, for a matrix A
%   and a column vector u, using Gauss quadrature. The parameter theta should
%   contain an estimate of the largest singular value of A.
%
%   fval = GAUSSCOSH(A,u,theta,opts,show) adds more information.
%	opts: additional options, see code.
%	show: if set to 1, displays additional information (default 0).
%
%   [fval info] = GAUSSCOSH(A,...) gets more information.
%	info: info(1) is always 0. info(2) contains the number of matrix
%	      vector products performed during the computation.

%   James Baglama, University of Rhode Island, U.S.A.
%   Caterina Fenu & Giuseppe Rodriguez, University of Cagliari, Italy
%   Lothar Reichel, Kent State University, U.S.A.
%   Email: jbaglama@math.uri.edu, reichel@math.kent.edu
%          kate.fenu@gmail.com, rodriguez@unica.it, 
%
%   Last revised Jun 30, 2014

if nargin < 5 || isempty(show), show = 0; end
if nargin < 4 || isempty(opts), opts = struct('empty','empty'); end
if nargin < 3, error('Too few input arguments.'); end

keepB = 0;
if nargout == 3, keepB = 1; end
if nargout > 3, opts.gaussreorth = 1; end

m = size(u,1);
if isa(A,'double')
	mvprod = @(x) A*x;
	mtvprod = @(x) A'*x;
elseif isa(A,'char') || isa(A,'function_handle') || isa(A,'inline')
	mvprod = @(x) A(x,'N');
	mtvprod = @(x) A(x,'T');
else
	error('A must be either a matrix or a function.')
end

nam = fieldnames(opts);
% tolerance for quadrature convergence
if ~any(strcmpi('gausstolq',nam)), opts.gausstolq = 1e-3; end
% tolerance for check if beta is 0
if ~any(strcmpi('gausstau',nam)), opts.gausstau = 1e-10; end
% max number of iterations
if ~any(strcmpi('gaussmaxn',nam)), opts.gaussmaxn = min(100,m); end
% value of mu for scaling expm, defaults to theta
if ~any(strcmpi('gaussmu',nam)), opts.gaussmu = theta; end
% 1 for reorth (see code)
if ~any(strcmpi('gaussreorth',nam)), opts.gaussreorth = 0; end
% 1 for mgs
if ~any(strcmpi('gaussmgs',nam)), opts.gaussmgs = 1; end

sigma = norm(u);

if abs(sigma - 1) > eps
	error('The vector u is not normalized.');
end

if opts.gaussreorth
	Q = zeros(m,opts.gaussmaxn+1);
	P = zeros(m,opts.gaussmaxn+1);
end

if keepB, B = zeros(opts.gaussmaxn+1,opts.gaussmaxn+1); end

T = zeros(opts.gaussmaxn+1,opts.gaussmaxn+1);
sigm = [sigma;zeros(opts.gaussmaxn,1)];
rh = zeros(opts.gaussmaxn,1);

q = u/sigma;
p = zeros(m,1);
if opts.gaussreorth, Q(:,1) = q;end

flag = 1;
g = 0;
i = 0;
info = 0;
%sqrteps = sqrt(eps);

while flag && (i<opts.gaussmaxn)

	i = i + 1;

	ptilde = mtvprod(q);
	ptilde = ptilde - sigma*p;

	% Reorthonormalization Columns of P
	if opts.gaussreorth
		switch opts.gaussreorth
			case 1	% MGS
				for j = 1:i-1
					ptilde = ptilde - (ptilde'*P(:,j))*P(:,j);
				end
			case 2	% CGS
				ptilde = ptilde - P(:,1:i-1)*(P(:,1:i-1)'*ptilde);
			case 3	% double CGS
				c = P(:,1:i-1)'*ptilde;
				ptilde = ptilde - P(:,1:i-1)*c;
				%alfa = alfa + sum(c);		% See irbleigs
				if norm(c) > sqrteps
					c1 = P(:,1:i-1)'*ptilde;
					ptilde = ptilde - P(:,1:i-1)*c1;
					%alfa = alfa + sum(c1);	% See irbleigs
				end
		end
	end

	rho = norm(ptilde);

	if rho < opts.gausstau
		if show
			warning('gausscosh:nullRho',...
				'Rho too small for i=%d, e_%d.',i,find(u));
		end
	else
		p = ptilde/rho;
	end

	if opts.gaussreorth, P(:,i) = p; end

	qtilde = mvprod(p);
	qtilde = qtilde - rho*q;

	% Reorthonormalization Columns of Q
	if opts.gaussreorth
		switch opts.gaussreorth
			case 1	% MGS
				for j = 1:i
					qtilde = qtilde - (qtilde'*Q(:,j))*Q(:,j);
				end
			case 2	% CGS
				qtilde = qtilde - Q(:,1:i)*(Q(:,1:i)'*qtilde);
			case 3	% double CGS
				c = Q(:,1:i)'*qtilde;
				qtilde = qtilde - Q(:,1:i)*c;
				%alfa = alfa + sum(c);		% See irbleigs
				if norm(c) > sqrteps
					c1 = Q(:,1:i)'*qtilde;
					qtilde = qtilde - Q(:,1:i)*c1;
					%alfa = alfa + sum(c1);	% See irbleigs
				end
		end
	end

	sigma = norm(qtilde);

	if sigma < opts.gausstau
		if show
			warning('gausscosh:nullSigma',...
				'Sigma too small for i=%d, e_%d.',i,find(u));
		end
		T(i,i) = T(i,i) + rho^2;
		SQ = sqrtm_fun(T(1:i,1:i));
		CSH = cosh_fun(SQ,opts.gaussmu);
		g = CSH(1,1);
		gr = g;
		break
	else
		q = qtilde/sigma;
	end

	if opts.gaussreorth, Q(:,i+1) = q; end

	rh(i) = rho;
	sigm(i+1) = sigma;

	if keepB
		B(i,i) = rho;
		B(i+1,i) = sigma;
	end

	%keyboard
	%norm1 = norm(A*(P(:,1:i))-(Q(:,1:i+1))*B(1:i+1,1:i),inf);
	%norm2 = norm(A'*(Q(:,1:i))-(P(:,1:i))*(B(1:i,1:i))',inf);
	%pause

	T(i:i+1,i:i+1) = T(i:i+1,i:i+1) + [rho^2 rho*sigma;rho*sigma 0];
	[pm pm1] = opolyval(diag(T(1:i,1:i)),diag(T(1:i+1,1:i+1),1),1,theta);
	T(i+1,i+1) = theta - (rho*sigma)*(pm1/pm);

	SQ = sqrtm_fun(T(1:i+1,1:i+1));

	CSH = cosh_fun(SQ,opts.gaussmu);
	gr = CSH(1,1);

	T(i+1,i+1) = sigma^2;
	SQ = sqrtm_fun(T(1:i,1:i));
	CSH = cosh_fun(SQ,opts.gaussmu);
	g = CSH(1,1);
	diff = abs(gr - g);

	%A*A'*Q(:,1:i) - Q(:,1:i)*T(1:i,1:i) -
	%B(i+1,i)*B(i,i)*Q(:,i+1)*[zeros(1,i-1) 1], pause

	%pause

	flag = diff > opts.gausstolq*abs(g);

end

fval = (gr + g)/2;
info = [info i];

e = exp(opts.gaussmu);
if isfinite(e)
	fval = e*fval;
else
	warning('gausscosh:scaledCentrality',...
		'Subgraph centrality is scaled by exp(mu).')
end


function S = sqrtm_fun(T)

[V D] = eig(T);
d = diag(D);
S = V*diag(sqrt(d))*V';


function C = cosh_fun(T,mu)

if nargin < 2, mu = 0; end

C = (expm_eig(T,mu) + expm_eig(-T,mu))/2;


function F = expm_eig(T,mu)

if nargin < 2, mu = 0; end

[V D] = eig(T);
d = diag(D);
F = V*diag(exp(d-mu))*V';


function [p p1] = opolyval(alfa,beta,beta0,x)
%OPOLYVAL computation of orthonormal polynomials
%   p=opolyval(a,b,beta0,x) computes p_{k+1}(x), where x is a vector and
%   p_{k+1} is the orhonomal polynomial defined by the recursion
%      b_k*p_{k+1}(x) = (x-alpha_k)*p_k(x) - b_{k-1}*p_{k-1}(x),
%   with p_0(x)=1/beta0 and p_{-1}(x)=0; beta0 is the norm of p(x)=1.
%   The coefficients a(1:k) and b(1:k) of the orthonormal polynomials can be
%   obtained by the coefficients rho(1:k) and gamma(1:k) of the monic
%   orthogonal polynomials, by taking a_i=rho_i and b_i=sqrt(gamma_i).
%   [p,p1]=opolyval(...) returns also the values of p_k(x).

x = x(:);

p1 = ones(size(x,1),1)/beta0;
p = (x-alfa(1))/beta(1);

for k = 2:size(alfa,1)
	p0 = p1;
	p1 = p;
	p = (x-alfa(k)).*p1 - beta(k-1).*p0;
	p = p/beta(k);
end

