function [viphh,vipha,hybcenh,hybcena,info,viph,vipa,viphcen,vipacen,infolr,LIST_H,LIST_A,Lh,La] = hubauthhyb(A,nnodes,opts,show,viph,vipa,viphcen,vipacen,infolr,LIST_H,LIST_A,Lh,La,mu)
%HUBAUTHHYB Hub/authority centrality computation via hybrid algorithm.
%   [nodeh nodea] = HUBAUTHHYB(A) identifies the most important nodes,
%   according to hub and authority centrality, of a network whose adjacency
%   matrix is A.
%
%   [nodesh nodesa] = HUBAUTHHYB(A,nnodes,opts,show) adds more information.
% 	nnodes: number of nodes to be identified (default 1).
%	opts: additional options, see code.
%	show: if set to 1, displays pictures and results (default 0).
%
%   [nodesh nodesa valsh valsa info] = HUBAUTHHYB(A,...) gets more information.
%	nodesh, nodesa: list of nodes.
%	valsh, valsa: approximation of hub/auth centrality for each node.
%	info: info(1) is a flag for the hub centrality and info(2) for the
%	      authority centrality: 0 means success, 1 means that the weak
%	      convergence test was satisfied (see code), 2 means failure.
%	      info(3) contains the number of matrix vector products performed
%	      during the computation.

%   James Baglama, University of Rhode Island, U.S.A.
%   Caterina Fenu & Giuseppe Rodriguez, University of Cagliari, Italy
%   Lothar Reichel, Kent State University, U.S.A.
%   Email: jbaglama@math.uri.edu, reichel@math.kent.edu
%          kate.fenu@gmail.com, rodriguez@unica.it, 
%
%   Last revised Ago 4, 2013

if nargin < 4 || isempty(show),  show = 0; end
if nargin < 3 || isempty(opts),  opts = struct('empty','empty');  end
if nargin < 2,  error('Too few input arguments.');  end

nam = fieldnames(opts);
% addtional nodes for weak convergence test
if ~any(strcmpi('sgcincr',nam)),  opts.sgcincr = 5;  end
% tolerance for weak convergence test in sgcenlr
if ~any(strcmpi('sgctoll',nam)),  opts.sgctoll = 1e-3;  end
% max number of iterations
if ~any(strcmpi('sgcnmax',nam)),  opts.sgcnmax = 100;  end
% tolerance for quadrature convergence
if ~any(strcmpi('gausstolq',nam)),  opts.gausstolq = 1e-3;  end

n = size(A,1);

if nargin < 5
	[viph vipa viphcen vipacen infolr LIST_H LIST_A Lh La mu] = hubauthlr(A,nnodes,opts,show);
end

info = infolr(1:2);	% 0 strong conv, 1 weak conv, 2 failure
mvp = infolr(4);

ei = zeros(n,1);
sizh = size(viph,1);
siza = size(vipa,1);

allhub = zeros(sizh,1);
allauth = zeros(siza,1);

itersh = zeros(sizh,1);
itersa = zeros(siza,1);

% for gausscosh
opts.gaussmu = mu;
theta = opts.gaussmu;
%opts.gaussreorth = 1;
%opts.gaussmgs = 1;
%opts.gausstheta = norm(A,inf);
%Afun = @(x) [A*x(n+1:2*n);A'*x(1:n)];

for i = 1:sizh
	ei(viph(i)) = 1;
	[allhub(i) infog] = gausscosh(A,ei,theta,opts,show);
	% gausscosh never fails so we don't need to check the flag
	itersh(i) = infog(2);
	mvp = mvp + infog(2);
	ei(viph(i)) = 0;
end

[sgsh isgsh] = sort(allhub,'descend');
viph = viph(isgsh);
allhub = allhub(isgsh);
viphh = viph(1:nnodes);
hybcenh = allhub(1:nnodes);

for i = 1:siza
	ei(vipa(i)) = 1;
	[allauth(i) infog] = gausscosh(A',ei,theta,opts,show);
	% gausscosh never fails so we don't need to check the flag
	itersa(i) = infog(2);
	mvp = mvp + infog(2);
	ei(vipa(i)) = 0;
end

[sgsa isgsa] = sort(allauth,'descend');
vipa = vipa(isgsa);
allauth = allauth(isgsa);
vipha = vipa(1:nnodes);
hybcena = allauth(1:nnodes);

info = [info mvp];

