function [viph vipa viphcen vipacen info LIST_H LIST_A Lh La mu MOV] = hubauthlr(A,nnodes,opts,show)
%HUBAUTHLR Hub/authority centrality computation via low rank approx.
%   [nodeh nodea] = HUBAUTHLR(A) identifies the two most important nodes, 
%   according to hub and authority centrality, of a network whose adjacency
%   matrix is A.
%
%   [nodesh nodesa] = HUBAUTHLR(A,nnodes,opts,show) adds more information.
% 	nnodes: number of nodes to be identified (default 1).
%	opts: additional options, see code.
%	show: if set to 1, displays pictures and results (default 0).
%
%   [nodesh nodesa valsh valsa info] = HUBAUTHLR(A,...) gets more information.
%	nodesh, nodesa: list of nodes.
%	valsh, valsa: approximation of hub/auth centrality for each node.
%	info: info(1) is a flag for the hub centrality and info(2) for the
%	      authority centrality: 0 means success, 1 means that the weak
%	      convergence test was satisfied (see code), 2 means failure.
%	      info(3:end) contains info from IRLBA_MOD or IRBLSVDS_LEJA.

%   James Baglama, University of Rhode Island, U.S.A.
%   Caterina Fenu & Giuseppe Rodriguez, University of Cagliari, Italy
%   Lothar Reichel, Kent State University, U.S.A.
%   Email: jbaglama@math.uri.edu, reichel@math.kent.edu
%          kate.fenu@gmail.com, rodriguez@unica.it, 
%
%   Last revised Ago 4, 2013

% number of VIP nodes to be found
if (nargin<2) || isempty(nnodes),  nnodes = 1;  end
% various options
if (nargin<3) || isempty(opts),  opts = struct('empty','empty');  end
% display intermediate results
if (nargin<4) || isempty(show),  show = 0;  end

% check options
nam = fieldnames(opts);
% max number of iterations
if ~any(strcmpi('sgcmaxn',nam)),  opts.sgcmaxn = 300;  end
% max number of columns for VA
if ~any(strcmpi('sgcmaxva',nam)),  opts.sgcmaxva = 200;  end
% addtional nodes for weak convergence test
%if ~any(strcmpi('sgcincr',nam)),  opts.sgcincr = 0;  end
% tolerance for weak convergence test
if ~any(strcmpi('sgctoll',nam)),  opts.sgctoll = 0;  end
% tau
if ~any(strcmpi('sgctau',nam)),  opts.sgctau = 1e-1;  end
% min number of incr
if ~any(strcmpi('sgcminkappa',nam)),  opts.sgcminkappa = 5;  end
% max number of incr
if ~any(strcmpi('sgcmaxkappa',nam)),  opts.sgcmaxkappa = 100;  end
% sgcshift = 1 for spectrum shift, 0 otherwise
if ~any(strcmpi('sgcshift',nam)),  opts.sgcshift = 1;  end

% the following are for irblsvds_leja
% number of singular values
if ~any(strcmpi('k',nam)),  opts.k = 5;  end
% blocksize
if ~any(strcmpi('blsz',nam)),  opts.blsz = 5;  end
% number of blocks
if ~any(strcmpi('nbls',nam))
	opts.nbls = ceil(opts.k/opts.blsz+1.1) + 2;
end
% 'LS', 'SS', numeric value
if ~any(strcmpi('sigma',nam)),  opts.sigma = 'LS';  end
% show intermediate results
if ~any(strcmpi('dispr',nam)),  opts.dispr = 0;  end
% maximum number of iterations
if ~any(strcmpi('maxit',nam)),  opts.maxit = 300;  end
% tolerance used for convergence
if ~any(strcmpi('tol',nam)),  opts.tol = 1e-3;  end

% options for irblsvds_leja
optsl.k = opts.k;
optsl.blsz = opts.blsz;
optsl.nbls = opts.nbls;
optsl.maxit = opts.maxit;
optsl.tol = opts.tol;

% for internal use
restype = 1;
makemovie = 0;  MOV = [];

n = size(A,1);
if nargout>5,  keepS = 1;  else  keepS = 0;  end
UA = [];	% left singular vector database
VA = [];	% right singular vectors database

kincr = opts.k;
opts.m_b = opts.k + 10; opts.ext = 1;
[U S V info F B] = irlba_mod(A,opts);
if info(1)
	warning('hubauthlr:irlba_mod','IRLBA_MOD did not converge.')
end
infosave = info;
sigma = diag(S);
[sigma vind] = sort(sigma,'descend'); % The singular values are not already sorted in decreasing order
U = U(:,vind);
V = V(:,vind);
numeig = opts.k;

if opts.sgcshift	% spectrum shift
	mu = sigma(1);
else
	mu = 0;
end

if show, figure(show), clf, figure(show+1), clf, end

Lh = zeros(n,1);
Uh = zeros(n,1);
La = zeros(n,1);
Ua = zeros(n,1);
usum = zeros(n,1);
vsum = zeros(n,1);
if keepS,  LIST_H = cell(numeig,1);  LIST_A = cell(numeig,1); end

N = 0;
realN1 = 0;
realN2 = 0;
realN = 0;
restart = 0;
flag = 1;
flag1 = 1;
flag2 = 1;
lish = [];
lisa = [];
while flag && (realN<min(opts.sgcmaxn,n)) && (N<numeig)

	N = N + 1;
	%csh = cosh(sigma(N));
	csh = (exp(sigma(N)-mu) + exp(-sigma(N)-mu)) / 2;

	if flag1
		realN1 = realN1+1;
		u = U(:,N).^2;
		usum = usum + u;
		Lh = Lh + csh*u;
		Uh = Lh + csh*(1-usum);
		[Lhs indxh] = sort(Lh,'descend');
		Lmaxh = Lhs(nnodes);		% Lmax = max(L);
		lish = find(Uh >= Lmaxh);
		flag1 = (numel(lish)>nnodes) && (csh/n>opts.sgctoll*Lmaxh);
		if keepS, LIST_H{realN1} = lish; end
		if show, rst1 = restart; end
	end
	if flag2
		realN2 = realN2+1;
		v = V(:,N).^2;
		vsum = vsum + v;
		La = La + csh*v;
		Ua = La + csh*(1-vsum);
		[Las indxa] = sort(La,'descend');
		Lmaxa = Las(nnodes);		% Lmax = max(L);
		lisa = find(Ua >= Lmaxa);
		flag2 = (numel(lisa)>nnodes) && (csh/n>opts.sgctoll*Lmaxa);
		if keepS, LIST_A{realN2} = lisa; end
		if show, rst2 = restart; end
	end

	%[flag1 numel(lish) csh/(Lmaxh*n)]
	%[flag2 numel(lisa) csh/(Lmaxa*n)]

	flag = flag1 || flag2;
	realN = max(realN1,realN2);

	if show

		figure(show)
		hold on
		hh1 = plot([Lh Uh]);
		ylv(1) = 1.5*min(Lh);
		ylv(2) = 1.5*max(Lh);
		xlim([1 n])
		ylim(ylv)
		title(sprintf('N=%d - #LIST=%d - Restart=%d\n', realN1, numel(lish), rst1),'fontsize',14)
		hold off
		drawnow

		figure(show+1)
		hold on
		hh2 = plot([La Ua]);
		ylv(1) = 1.5*min(La);
		ylv(2) = 1.5*max(La);
		xlim([1 n])
		ylim(ylv)
		title(sprintf('N=%d - #LIST=%d - Restart=%d\n', realN2, numel(lisa), rst2),'fontsize',14)
		hold off
		drawnow

		if makemovie
			delete(ht)
			ypos = .8 * (ylv(2)-ylv(1)) + ylv(1);
			ht = text(50,ypos,sprintf('N=%d - #LIST=%d\n', ...
				realN, numel(lis)), 'fontsize',16);
			MOV(N) = getframe;
		end
	end

	if flag && (N == numeig)		% restart

		% switch to other restart if VA is too large
		if size(VA,2)+opts.k >= opts.sgcmaxva,  restype = 2;  end
		%fprintf('restype = %d\n', restype);

		switch restype
			case 1
				UA = [UA U];
				VA = [VA V];
				opts.UA = UA;
				opts.VA = VA;
				opts.F = F;
				opts.B = B;
				opts.k = opts.k + kincr;
				opts.m_b = opts.k + 10;
				opts.ext = 0;
				opts.v = V; opts.u = U;
				[U S V info F B] = irlba_mod(A,opts);
				if info(1)
					warning('hubauthlr:irlba_mod',...
						'IRLBA_MOD did not converge.')
				end
				infosave = infosave + info;
				sigma = diag(S);
				sigma = sigma(end-kincr:end);
				[sigma vind] = sort(sigma,'descend');
				U = U(:,vind);
				V = V(:,vind);
				if keepS
					LIST_H = [LIST_H; cell(numeig,1)];
					LIST_A = [LIST_A; cell(numeig,1)];
				end
				restart = restart + 1;
				N = 0;
			case 2
				optsl.sigma = sigma(N);

				[U S V info] = irblsvds_leja(A,optsl);
				if info(1)
					warning('hubauthlr:irblsvds_leja',...
						'IRBLSVDS_LEJA did not converge.')
				end
				infosave = infosave + info;

				%[U S V info] = svds(A,optsl.k,optsl.sigma);
				%infosave(1) = infosave(1) + info;
				%if info(1)
				%	warning('hubauthlr:svds',...
				%		'SVDS did not converge.')
				%end

				sigma = diag(S);
				[sigma vind] = sort(sigma,'descend');
				disp([size(sigma) realN N size(U) size(vind)])
				if size(U,1)==0, keyboard, end
				U = U(:,vind);
				V = V(:,vind);
				[ll ill] = min(abs(sigma-optsl.sigma));
				if keepS
					LIST_H = [LIST_H; cell(numeig-ill,1)];
					LIST_A = [LIST_A; cell(numeig-ill,1)];
				end
				restart = restart + 1;
				N = ill;
			otherwise
				error('wrong restart')
		end
	end
end

if keepS,  LIST_H = LIST_H(1:realN1);  LIST_A = LIST_A(1:realN2);  end

info = [0 0 infosave];

if flag1
	warning('hubauthlr:tooFewSingVals',...
		'too few singular values to identify HUB nodes\n')
	info(1) = 2;
	nviph = numel(lish);
elseif numel(lish) > nnodes
	ivech = find((Lmaxh-Lhs(nnodes+1:end))<opts.sgctau*Lmaxh);
	incrh = min(length(ivech),opts.sgcmaxkappa);
	if incrh < opts.sgcminkappa, incrh = opts.sgcminkappa; end
	nviph = nnodes+incrh;
	info(1) = 1;
else
	nviph = nnodes;
end

if flag2
	warning('hubauthlr:tooFewSingVals',...
		'too few singular values to identify AUTHORITY nodes\n')
	info(2) = 2;
	nvipa = numel(lisa);
elseif numel(lisa) > nnodes
	iveca = find((Lmaxa-Las(nnodes+1:end))<opts.sgctau*Lmaxa);
	incra = min(length(iveca),opts.sgcmaxkappa);
	if incra < opts.sgcminkappa, incra = opts.sgcminkappa; end
	nvipa = nnodes+incra;
	info(2) = 1;
else
	nvipa = nnodes;
end

viph = indxh(1:nviph);
vipa = indxa(1:nvipa);
viphcen = Lhs(1:nviph);
vipacen = Las(1:nvipa);

e = exp(mu);
if isfinite(e)
	viphcen = e*viphcen;
	vipacen = e*vipacen;
	if nargout > 7
		Lh = e*Lh;
		La = e*La;
	end
else
	warning('hubauthlr:scaledCentrality',...
		'Hub and Authority centrality are scaled by exp(mu)')
end

if show
	fprintf('\n\n*******HUB**********\n\n')
	switch info(1)
		case 0
			fprintf('Strong convergence test was satisfied.\n')
		case 1
			fprintf('Weak convergence test was satisfied.\n')
		case 2
			fprintf('No convergence test was satisfied.\n')
	end

	figure(show)
	hold on
	plot(viph,viphcen,'r.','markersize',20)
	hold off
	drawnow

	fprintf('\n\n*******AUTHORITY**********\n\n')
	switch info(2)
		case 0
			fprintf('Strong convergence test was satisfied.\n')
		case 1
			fprintf('Weak convergence test was satisfied.\n')
		case 2
			fprintf('No convergence test was satisfied.\n')
	end

	figure(show+1)
	hold on
	plot(vipa,vipacen,'r.','markersize',20)
	hold off
	drawnow
end

