function varargout = irblsvds_leja(varargin)
%
global plotexampleleja plotexampleblk plotexamplelejarh;
%
% IRBLSVDS_LEJA will find a few extreme or interior singular values and singular vectors 
% of an M x N matrix A such that A*V = U*S, where V is an orthonormal n x k matrix of "right" singular 
% vectors, U is an orthonormal m x k matrix of "left" singular vectors, and S is 
% a k x k diagonal matrix of singular values. IRBLSVDS_LEJA  uses an implicitly shifted block
% Lanczos bidiagonalization method with Leja points as shifts.
%
% [U,S,V,FLAG] = IRBLSVDS(A,OPTIONS) 
% [U,S,V,FLAG] = IRBLSVDS('Afunc',M,N,OPTIONS) 
%
% The first input argument into IRBLSVDS can be a numeric matrix A or an M-file 
% ('Afunc'). If the M x N matrix A is a filename then Y = Afunc(X,M,N,BLSZ,'transpose'). 
% If transpose = 'F', then X is an (N x blsz) matrix and Y = A*X. If transpose = 'T', 
% then X is an (M x BLSZ) matrix and Y = A'*X.
% 
% OUTPUT OPTIONS:
% ---------------
%
% I.)   IRBLSVDS_LEJA(A) 
%       Displays the desired singular values.    
%
% II.)  S = IRBLSVDS_LEJA(A) 
%       Returns the desired singular values in the vector S. 
%
% III.) [U,S,V] = IRBLSVDS_LEJA(A)
%       S is a diagonal matrix that contains the desired singular values in descending
%       order along the diagonal, the matrix V contains the corresponding "right" singular
%       vectors, and U contains the corresponding "left" singular vectors such that 
%       that A*V = U*S, V'*V = I, and U'*U = I.  
%
% IV.)  [U,S,V,FLAG] = IRBLSVDS_LEJA(A)
%       This option returns the same as (III) plus a two dimensional array FLAG 
%       that reports if the algorithm converges and the number of matrix vector 
%       products. If FLAG(1) = 0 then this implies normal return: all desired 
%       singular values have converged. If FLAG(1) = 1 then the maximum number 
%       of iterations have been reached before all desired values converged. 
%       FLAG(2) contains the number of matrix vector products (A*X and A'*X) 
%       used by the code. If the maximum number of iterations are reached 
%       (FLAG(1) = 1), then the matrices U, V, and S contain any singular pairs
%       that have converged plus the last singular pair approximation for the 
%       pairs that have not converged.
%
% INPUT OPTIONS:
% --------------
%                                   
%       ... = IRBLSVDS_LEJA(A,OPTS)
%       OPTS is a structure containing input parameters. The input parameters can
%       be given in any order. The structure OPTS may contain some or all of the 
%       following input parameters. The string for the input parameters can contain
%       upper or lower case characters. 
%       
%  INPUT PARAMETER      DESCRIPTION                     
%   
%  OPTS.BLSZ         Block size of the Lanczos bidiagonal matrix.        
%                    DEFAULT VALUE    BLSZ = 3
%                        
%  OPTS.DISPS        Indicates if K approximate singular values and residuals are to be 
%                    displayed on each iteration. Set positive to display the values.
%                    DEFAULT VALUE   DISPS = 0 
%
%  OPTS.ENDPT        Three letter string specifying the location of the interior end- 
%                    points for the dampening interval(s). Intervals are located in
%                    the spectrum of A'A.           
%                    'FLT' - Let the interior endpoints float.                  
%                    'MON' - Interior endpoints are chosen so that the size of the 
%                            dampening interval is increasing. This creates a nested
%                            sequence of intervals. 
%                    DEFAULT VALUE   ENDPT = 'FLT' 
%
%  OPTS.K            Number of desired singular values.             
%                    DEFAULT VALUE  K = 6
%
%  OPTS.MAXIT        Maximum number of iterations, i.e. maximum number of restarts.                           
%                    DEFAULT VALUE  MAXIT = 1000
%
%  OPTS.MAXDPOL      Numeric value indicating the maximum degree of the dampening 
%                    polynomial allowed.  
%                    DEFAULT VALUE   MAXDPOL = 200        (If SIGMA = 'LS' or SIGMA = 'SS'.)
%                    DEFAULT VALUE   MAXDPOL = MAX(M,N)   (If SIGMA = a numeric value NVAL.)
%
%  OPTS.NBLS         Number of blocks in the Lanczos bidiagonal matrix. The program may increase
%                    NBLS to ensure certain requirements are satisfied. A warning message
%                    will be displayed if NBLS increases.                          
%                    DEFAULT VALUE    NBLS = 5
%
%  OPTS.REORTH       Three letter string specifying whether to use one-sided full reorthogonalization
%                    or two-sided. One-sided is performed only on the "short" vectors.
%                    Two-sided orthogonality is used when cond(A) estimated by cond(B) > 1/sqrt(eps).
%                    'ONE'  One-sided full reorthogonalization.
%                    'TWO'  Two-sided full reorthogonalization.
%                    DEFAULT VALUE  REORTH = 'ONE'
%
%  OPTS.SIGMA        Two letter string specifying the location of the desired singular values.            
%                    'LS'  Largest singular values.
%                    'SS'  Smallest singular values.
%                    'CN'  Condition number of A.
%                    NVAL  A numeric value. The program searches for the K closest
%                          singular values to the numeric value NVAL. NVAL>
%                          0. If NVAL <= 0 SIGMA is set to 'SS'.
%                    DEFAULT VALUE   SIGMA = 'LS'
%                                                         
%  OPTS.SIZINT       Size of the dampening interval. 
%                    DEFAULT VALUE    SIZINT = 1
%                                 
%  OPTS.TOL          Tolerance used for convergence. Convergence is determined when             
%                    || A*V - U*S ||_2 <= TOL*||A||. Where ||A|| is approximated by                                   
%                    the largest singular value of all projection matrices.  
%                    DEFAULT VALUE    TOL = 1d-6 
%                                                              
%  OPTS.V0           A matrix of approximate right singular vectors, if M >= N
%                    and sigma = 'LS', 'SS' or NVAL. If M < N and sigma = 'SS' then
%                    V0 must be a matrix of left singular vectors. This avoids
%                    computing zero eigenvalues of A'A. 
%                    DEFAULT VALUE  V0 = []

%
%  OPTS.ZERTYP       Two letter string to indicate which type of zeros to apply. Leja points
%                    are choosen to dampen the spectrum of A'A.             
%                    'WL' - Weighted fast Leja points. The weight functions are used to help
%                           increase convergence.  
%                    'ML' - Mapped fast Leja points. Fast Leja points are computed on [-2,2] 
%                           and mapped to the dampening interval. This option is not available
%                           when sigma is a numeric value NVAL.
%                    DEFAULT VALUE  ZERTYP = 'ML' 
%
% 
%  DATE:  3/24/11
%  VER:  2.0

%  AUTHORS:
%  James Baglama     University of Rhode Island, E-mail: jbaglama@math.uri.edu
%  Lothar Reichel    Kent State University, E-mail: reichel@mcs.kent.edu
   
% REFERENCES:
%   1.)  J. Baglama and L. Reichel, "IRBLB: Implicitly Restarted Block Lanczos Bidiagonalization 
%        Method with Leja Shifts", Technical report, 2011.
%   2.)  J. Baglama and L. Reichel, "Restarted Block Lanczos Bidiagonalization Methods"
%        Numerical Algorithms, 43 (2006), pp. 251-272.
%   3.)  J. Baglama and L. Reichel, "Augmented Implicitly Restarted Lanczos
%        Bidiagonalization Methods"SIAM J. Sci. Comput., 27 (2005), pp. 19-42. 
%   4.)  J. Baglama, D. Calvetti, and L. Reichel, "IRBL: An Implicitly Restarted Block 
%        Lanczos Method for large-scale Hermitian eigenproblems", SIAM J. Sci. Comput., 
%        24, No. 5, (2003), pp. 1650-1677
%   5.)  J. Baglama, D. Calvetti, and L. Reichel,"irbleigs: A MATLAB program for computing 
%        a few eigenpairs of a large sparse Hermitian matrix",TOMS, Vol.
%        29, No. 5, (2003), pp. 337-348.
%   6.)  J. Baglama, D. Calvetti, and L. Reichel"Fast Leja Points", ETNA, 
%        Vol. 7 (1998), pp. 124-140.
%   7.)  J. Baglama, D. Calvetti, and L. Reichel,"Iterative Methods for the Computation of 
%        a Few Eigenvalues of a Large Symmetric Matrix", BIT, 36 (1996), pp. 400-421.

% Incorrect number of output arguments requested.
if (nargout > 4 | nargout==2 ) error('ERROR: Incorrect number of output arguements.'); end

%----------------------------%
% BEGIN: PARSE INPUT VALUES. %
%----------------------------%

% No input arguments, return help.
if nargin == 0, help irblsvds_leja, return, end

% Matrix A is stored in varargin{1}. Check type (numeric or character) and dimensions.
if (isstruct(varargin{1})), error('A must be a matrix.'), end
if ischar(varargin{1})
   if nargin == 1, error('Need dimension M for the (M x N) matrix A).'), end  
   m = varargin{2};
   if ~isnumeric(m) | length(m) ~= 1 
      error('Second argument M must be a numeric value.'); 
   end
   if nargin == 2, error('Need dimension N for the (M x N) matrix A).'); end  
   n = varargin{3}; 
   if ~isnumeric(n) | length(n) ~= 1 
      error('Third argument N must be a numeric value.'); 
   end
else
   [m,n] = size(varargin{1});
end

% Set all input options to default values.
blsz = 3; disps = 0;  endpt = 'FLT'; K = 6; maxdpol = 200; maxit = 1000; nbls = 5; 
nval=[]; reorth='ONE'; sigma = 'LS'; sizint = 1; tol = 1d-6; V = []; zertyp = 'WL';
condnum = 0;

% Set indicators for ENDPTS and MAXDPOL to be empty arrays. The indicators determine which
% default values should be used.
IENDPT = []; IMAXDPOL = [];

% Get input options from the structure.
if nargin > 1 + 2*ischar(varargin{1})
   options = varargin{2+2*ischar(varargin{1}):nargin};
   names = fieldnames(options);
   I = strmatch('BLSZ',upper(names),'exact');
   if ~isempty(I), blsz = getfield(options,names{I}); end
   I = strmatch('DISPS',upper(names),'exact');
   if ~isempty(I), disps = getfield(options,names{I}); end
   I = strmatch('ENDPT',upper(names),'exact'); IENDPT = I;
   if ~isempty(I), endpt = upper(getfield(options,names{I})); end
   I = strmatch('K',upper(names),'exact');
   if  ~isempty(I), K = getfield(options,names{I}); end
   I = strmatch('MAXDPOL',upper(names),'exact'); IMAXDPOL = I;
   if ~isempty(I), maxdpol = getfield(options,names{I}); end
   I = strmatch('MAXIT',upper(names),'exact');
   if ~isempty(I), maxit = getfield(options,names{I}); end
   I = strmatch('NBLS',upper(names),'exact');
   if ~isempty(I), nbls = getfield(options,names{I}); end
   I = strmatch('ZERTYP',upper(names),'exact');
   if ~isempty(I), zertyp = upper(getfield(options,names{I})); end
   I = strmatch('REORTH',upper(names),'exact');
   if ~isempty(I), reorth = upper(getfield(options,names{I})); end
   I = strmatch('SIGMA',upper(names),'exact');
   if  ~isempty(I), sigma = upper(getfield(options,names{I})); end
   I = strmatch('SIZINT',upper(names),'exact');
   if ~isempty(I), sizint = getfield(options,names{I}); end
   I = strmatch('TOL',upper(names),'exact');
   if ~isempty(I), tol = getfield(options,names{I}); end
   I = strmatch('V0',upper(names),'exact');
   if ~isempty(I), V = getfield(options,names{I}); end
end 
 

% Check type of input values and output error message if needed.
if (~isnumeric(blsz) || ~isnumeric(disps)   || ~ischar(endpt)     || ... 
    ~isnumeric(K)    || ~isnumeric(maxdpol) || ~isnumeric(maxit)  || ...
    ~isnumeric(nbls) || ~ischar(zertyp)     || ~isnumeric(sizint) || ...
    ~isnumeric(tol))
   error('Incorrect type for input value(s) in the structure.');
end

% If a numeric value is given for sigma then set nval=sigma and sigma = 'IE' to
% denote that the code is searching for interior eigenvalues. if nval <= 0
% then set sigma=SS and set nval=[].
if isnumeric(sigma),  
    nval = sigma;
    sigma = 'IS'; 
    if nval <=0, sigma='SS'; nval=[]; end
end

% Check the length of the character values sigma, endpt, and zertyp.
if length(sigma)  ~= 2, error('SIGMA must be LS, SS, CN, or a numerical value'); end
if length(endpt)  ~= 3, error('Incorrect value for ENDPT');  end
if length(zertyp) ~= 2, error('Incorrect value for ZERTYP'); end
 

% Interchange m and n so that size(A'A) = min(m,n). Avoids
% finding zero values when searching for the smallest singular
% values.
interchange = 0; if n > m && (strcmp(sigma,'SS') || strcmp(sigma,'CN')), t=m; m=n; n=t; interchange = 1; end 


% For condition number calculuations only. Set K =2, smallest and largest
% singular value.
if strcmp(sigma,'CN'), K  = 2; condnum = 1; disps=0; end

% Check for input errors in the data structure.
%if sizint  <  1,        error('Incorrect value for SIZINT. SIZINT must be >= 1.'), end
if K       <= 0,        error('K must be a positive value.'),    end
if K       >  min(n,m), error('K must be less than min(n,m).'),  end   
if blsz    <= 0,        error('BLSZ must be a positive value.'), end
if nbls    <= 1,        error('NBLS must be greater than 1.'),   end
if tol     <  0,        error('TOL must be non-negative.'),      end
if maxit   <= 0,        error('MAXIT must be positive.'),        end
if maxdpol < nbls,      error('MAXDPOL must be >= NBLS'),        end
%if blsz*nbls - sizint <= blsz, error('SIZINT is too large'),     end
%if blsz*nbls - K - blsz - sizint < 0
%   nbls = ceil((K+sizint+blsz)/blsz+0.1); 
%   warning(['Increasing NBLS to ',num2str(nbls)]);
%end
if blsz*nbls >= min(n,m), error('K or SIZINT are too large.'), end
if (~strcmp(sigma,'SS') && ~strcmp(sigma,'LS') && ~strcmp(sigma,'IS') && ~strcmp(sigma,'CN'))
   error('SIGMA must be LS, SS, CN, numerical value.'); 
end
if (~strcmp(endpt,'FLT') && ~strcmp(endpt,'MON')), error('Unknown value for ENDPT.'),  end
if (~strcmp(zertyp,'WL') && ~strcmp(zertyp,'ML')), error('Unknown value for ZERTYP.'), end
if strcmp(sigma,'IS') || strcmp(sigma,'CN') 
   zertyp = 'WL'; 
   if isempty(IENDPT),   endpt = 'FLT'; end
   if isempty(IMAXDPOL), maxdpol = max(n,m);   end
end

% Preallocate memory for W and F. These matrices are full and resizing will cause
% an increase in cpu time.
W = zeros(m,blsz*nbls); F = zeros(n,blsz);

% If starting matrix V0 is not given then set starting matrix V0 to be a 
% (n x 1) matrix of normally distributed random numbers.
if isempty(V)
  V = zeros(n,blsz*nbls); % Preallocate memory for V. This matrix is a full matrix.
  V(:,1:blsz) = randn(n,blsz); 
else 
  if (size(V,2) ~= blsz), error('Incorrect size of starting matrix V0.'), end
  V(:,blsz+1:blsz*nbls) = zeros(n,blsz*nbls-blsz);
end
if ~isnumeric(V), error('Incorrect starting matrix V0.'), end
if (size(V,1) ~= n), error('Incorrect size of starting matrix V0.'), end  
 
% Set tolerance to machine precision if tol < eps.
if tol < eps, tol = eps; end

%--------------------------%
% END: PARSE INPUT VALUES. %
%--------------------------%

%-----------------------------------------------------------%
% BEGIN: DESCRIPTION AND INITIALIZATION OF LOCAL VARIABLES. %
%-----------------------------------------------------------%

% Initialization and description of local variables.
B = [];              % Block bidiagonal matrix.
Bsz = blsz*nbls;     % Size of the block bidiagonal matrix.
conv = 0;            % Number of desired singular values that have converged.
chsigma = 0;         % Sigma has been changed. Used in Condition number finding.
deflate = 0;         % Value used to determine if an undesired singular vector has converged.
eps23 = eps^(2/3);   % Two thirds of eps, used for Smax. Avoids using zero.        
flcount = 0;         % Count used to determine when the maximum number of shifts is reached.
fLejapts = [];       % Stores fast Leja points.
hritzs =[];          % Harmonic Ritz of the shifted B'B - nval. Only need if SIGMA is numerical value.
iter = 1;            % Main loop iteration count.
lcandpts = [];       % Stores "left" candidate points.
lindex = [];         % Index for lcandpts.
lprd = [];           % Stores the Leja polynomial product for "left" candidate points.
leftendpt = [];      % Stores the left most endpoint of the dampening interval.
leftintendpt = [];   % Stores the interior left endpoint of the dampening interval.
leftLejapt = [];     % Place holder in fast leja routines.
mprod = 0;           % The number of matrix vector products.
norlpol = [];        % Normalizes the Leja polynomial to avoid underflow/overflow.
numbls = nbls;       % Initial size of the Lanczos blocks.
rcandpts = [];       % Stores "right" candidate points.
rindex = [];         % Index for rcandpts.
ritz=[];             % Singular values of S squared. 
rprd = [];           % Stores the Leja polynomial product for "right" candidate points.
rightendpt = [];     % Stores the right most endpoint of the dampening interval.
rightintendpt = [];  % Stores the interior right endpoint of the dampening interval.
rightLejapt = [];    % Place holder in fast Leja routines.
S_B =[];             % Singular values of B.
U_B =[];             % Left singular vectors of B.
V_B =[];             % Right singular vectors of B.
singblk = [];        % Integer value used to indicate singular block(s) in blanzbd.
singconv = 'F';      % Boolean to determine if all desired singular values have converged.
singresdisp = [];    % Holds the residual values of converged singular values.
singval = [];        % Array used to hold the converged desired singular values.
Smax = [];           % Holds the maximum value of all computed singular values of B.          
sqrteps = sqrt(eps);  % Square root of machine tolerance used in convergence testing.
SVTol = min(sqrteps,tol);  % Tolerance to determine when a singular vector has converged.
SVTol = eps;
V_A=[];              % Holds all converged right singular vectors.
U_A=[];              % Holds all converged left singular vectors.

% Determine if singular vectors are requested.
if (nargout > 1), computvec = 'T'; else computvec = 'F'; end 


%--------------------------------------------------------------------%
% END: DESCRIPTION AND INITIALIZATION OF LOCAL AND GLOBAL VARIABLES. %
%--------------------------------------------------------------------%

%----------------------------%
% BEGIN: MAIN ITERATION LOOP %
%----------------------------%

while (iter <= maxit)
   
   % Check and deflate the number of Lanczos blocks if possible. Do not
   % deflate if searching for interior or condition number.  
   if (nbls > 2 && ~strcmp(sigma,'IS') && ~condnum)
      nbls = numbls - floor(size(V_A,2)/blsz);
      if (sizint >= nbls*blsz-(abs(K-size(V_A,2))))
         nbls = max(floor((sizint + abs(K-size(V_A,2)))/blsz) + 1,numbls);
      end
  end   
   if nbls ~= numbls
         % Resize V and W.
         V = [V(:,1:blsz) zeros(n,blsz*nbls-blsz)]; 
         W = zeros(m,blsz*nbls);
   end      
   

   
   % Compute the Lanczos bidiagonalization decomposition.
   [V,W,F,B,mprod,singblk] = ablanzbd(varargin{1},V,V_A,W,U_A,F,B,interchange,nbls,blsz,n,...
                                    m,mprod,SVTol,reorth,iter,singblk); 
                                
                                
   
   % Check for problems and compute the size of B.
   Bsz = size(B,1); if Bsz ~= nbls*blsz, error('Incorrect size of B.'), end
                                
   % Compute singular triplets of B. MATLAB's svds orders the singular values
   % largest to smallest.
   [U_B,S_B,V_B] = svd(B); S_B = diag(S_B); ritz = (S_B).^2;
    
   % Estimate ||A|| using the largest singular value over all iterations
   % and estimate the cond(A) using approximations to the largest and smallest 
   % singular values. If a small singular value is less than sqrteps use only Ritz
   % vectors to augment and require two-sided reorthogonalization, i.e. set REORTH = 'TWO'.
    if isempty(Smax)
       Smax = S_B(1); Smin = S_B(Bsz);
    else 
       Smax = max(Smax,S_B(1)); Smin = min(Smin,S_B(Bsz));
    end
    Smax = max(eps23,Smax); 
    if Smin/Smax < sqrteps 
        reorth = 'TWO'; % Automatic switch reorthogonalization to two sided if ill-conditioned.
    end
    
   % Order the Ritz and singular values smallest to largest.
   [ritz,J] = sort(real(ritz)); V_B = V_B(:,J); U_B = U_B(:,J); S_B = S_B(J);

   % For condition number calculuations only.
   if strcmp(sigma,'CN'), nval = (S_B(1)^2+S_B(Bsz)^2)/2; end;
   
   % Reached maximum number of iterations, exit main loop.
   if iter >= maxit, break, end;
  
   % Compute the residuals for all singular values.
   residuals = sqrt(sum((F*U_B(Bsz-(blsz-1):Bsz,:)).^2,1));
    
   % Convergence tests.
   [conv,deflate,singval,V_A,U_A,sigma,chsigma,singresdisp,singconv,singblk,Smax,sizint,K,Smin,rhres] = convtests(computvec,conv,deflate, ...
    disps,singval,V_A,U_A,singresdisp,iter,K,residuals,S_B,singconv,U_B,V_B,B,sigma,...
    chsigma,nval,tol,SVTol,Bsz,V,W,singblk,Smax,F,blsz,varargin{1},sizint,Smin,condnum);


   plotexampleleja(iter,1:2) = [iter residuals(1)];
   plotexamplelejarh(iter,1:2) = [iter rhres(1) ];
   

   % If all desired singular values converged then exit main loop.
   if strcmp(singconv,'T'), break, end;
   
   if chsigma,  leftintendpt = []; rightintendpt = []; ...
               leftendpt = []; rightendpt = []; chsigma = 0; K = 1; conv = 0; end
       
   
   % Determine dampening intervals, Leja points, and apply Leja zeros as shifts.
   [fLejapts,lcandpts,leftendpt,leftintendpt,lprd,leftLejapt,lindex,maxdpol,nbls,norlpol,...
         nval,rcandpts,rightendpt,rightintendpt,rprd,rightLejapt,rindex,V,flcount] =... 
         applyshifts(blsz,endpt,F,fLejapts,K,lcandpts,leftendpt,leftintendpt,...
                     lprd,leftLejapt,lindex,maxdpol,nbls,norlpol,rcandpts,...
                     rightendpt,rightintendpt,ritz,V_B,rprd,rightLejapt,rindex,...
                     zertyp,sigma,nval,singblk,sizint,Bsz,B,V,flcount,sqrteps,residuals);
      
   % Check to see if an undesired singular vector has been chosen to be deflated. If so,
   % reset the endpoints of the dampening interval(s) on the next interation.
   if deflate, deflate = 0;  leftintendpt = []; rightintendpt = []; ...
               leftendpt = []; rightendpt = []; end
       
   
   % Update the main iteration loop count.
   iter = iter + 1;
   
   
   
end % While loop. 

%--------------------------%
% END: MAIN ITERATION LOOP %
%--------------------------%

%-----------------------%
% BEGIN: OUTPUT RESULTS %
%-----------------------%

% Test to see if maximum number of iterations is reached.
% If output options 1 or 2 are used then set singval, V_A,
% and U_A to be empty arrays.If output option 4 is used 
% then use the triplets on the last iteration as estimate 
% for the unconverged singular triplets.

LEJA_CODE = mprod;
%iter
%maxit

% Output for condition number only.
if condnum
    if ((nargout == 0) || (nargout > 1)), 
       ConditionNumber = Smax/Smin;
    end  
    if (nargout == 1),  varargout{1} = Smax/Smin; end 
    return;
end    
    
 
% Output
FLAG(1) = 0; 
if iter >= maxit && nargout <= 2
   FLAG(1) = 1; %singval = []; V_A = []; U_A=[];
end        
if iter >= maxit && nargout == 4
   FLAG(1) = 1;
   NC = K - length(singval);
   if strcmp(sigma,'SS'), JI = 1:NC; end
   if strcmp(sigma,'LS'), JI = Bsz-NC+1:Bsz; end
   if strcmp(sigma,'IS'), [sortval,JI] = sort(abs(S_B-nval)); end
   singval = [singval;S_B(JI(1:NC))];  
   V_A = [V_A,V*V_B(:,JI(1:NC))];
   U_A = [U_A,W*U_B(:,JI(1:NC))];
end   
  
% Sort output arguments.
if ~isempty(singval)
   K = min(length(singval),K); singval = real(singval);
   if strcmp(sigma,'SS'), [sortval,I] = sort(singval);  I = I(1:K); end   
   if strcmp(sigma,'LS')
      [sortval,I] = sort(singval); I = flipud(I); I = I(1:K);
   end
   if strcmp(sigma,'IS')
      [sortval,I] = sort(abs(singval-nval)); I = I(1:K);
      [sortval,J] = sort(singval(I)); I = I(J);
   end
   singval = singval(I);
end  

% Output option I: Display singular values only.
if (nargout == 0), 
  if iter >= maxit, disp('Maximum number of iterations exceeded.'); end
  SingularValues = singval;
end

% Output option II: Set singular values equal to output vector.  
if (nargout == 1),  varargout{1} = singval; end 

% Output option III and IV: Output singular triplets (U,S,V), where
% A*V = U*S.
if nargout > 1
   if isempty(singval), singval = S_B(1:K); end;
   if ~isempty(V_A)
      V_A = V_A(:,I); U_A = U_A(:,I);
   else
      V_A = []; U_A=[]; 
   end 
   if interchange
      varargout{1} = V_A; varargout{2} = diag(singval); varargout{3}=U_A;
   else
      varargout{1} = U_A; varargout{2} = diag(singval); varargout{3}=V_A;
   end
   if nargout == 4, FLAG(2) = mprod; varargout{4} = FLAG; end
end

%---------------------%
% END: OUTPUT RESULTS %
%---------------------%

%------------------------------------------------%
% BEGIN: LANCZOS BIDIAGONALIZATION DECOMPOSITION %
%------------------------------------------------%

function [V,W,F,B,mprod,singblk] = ablanzbd(A,V,V_A,W,U_A,F,B,interchange,nbls,blsz,n,...
                                    m,mprod,SVTol,reorth,iter,singblk)
% Computes the block Lanczos bidiagonalization decomposition
%  
%  A*V  = W*B
%  A'*W = V*B' + F*E^T
%  
% with full reorthogonalization. The matrix A can be passed as a numeric 
% matrix or as a filename. If the M x N matrix A is a filename then
% Y = Afunc(X,M,N,BLSZ,'transpose'). If transpose = 'F', then X is an (N x BLSZ)
% matrix and Y = A*X. If transpose = 'T', then X is an (M x BLSZ) matrix
% and Y = A'*X.

% James Baglama
% DATE: 3/15/2006

% Initialization of main loop count J, matrix B, and singblk.
J = 1:blsz; B=[]; singblk = []; singblk(1) = 0;

% Orthogonalize V against all converged right singular vectors V_A.
if ~isempty(V_A), V(:,J) = orthog(V(:,J),V_A,SVTol); end

% Compute the QR factorization of V.
[V(:,J),R] = qr(V(:,J),0); 
    
% Check for linearly dependent vectors.
R_max = max(abs(diag(R))); R_min=min(abs(diag(R)));
stol = max(SVTol,eps*R_max);
if R_max > stol
   rat_diag = R_min/R_max;
else
   rat_diag = 0; % All vectors in the block are numerically zero.
end 
if rat_diag <= stol
   if ~isempty(V_A), V(:,J) = orthog(V(:,J),V_A,SVTol); end 
   [V(:,J),R_2] = qr(V(:,J),0); R = R_2*R;
   singblk(1) = 1; % Set the singular block indicator to true.
end

% Matrix A product with vector(s) V, (W=A*V).
if interchange
   if ischar(A)
     W(:,J) = feval(A,V(:,J),m,n,blsz,'T');
  else
     W(:,J) = (V(:,J)'*A)';
  end
else
  if ischar(A)
     W(:,J) = feval(A,V(:,J),m,n,blsz,'F');
  else
     W(:,J) = A*V(:,J);
  end
end

% Count the number of matrix vector products.
mprod = mprod + blsz;

% Orthogonalize W against all converged left singular vectors U_A.
if ~isempty(U_A) && strcmp(reorth,'TWO'), W(:,J) = orthog(W(:,J),U_A,SVTol); end

% Compute the QR factorization of W.
[W(:,J),S] = qr(W(:,J),0); 

% Check for linearly dependent vectors.
S_max = max(abs(diag(S))); S_min=min(abs(diag(S)));
stol = max(SVTol,eps*S_max);
if S_max > stol
   rat_diag = S_min/S_max;
else
   rat_diag = 0; % All vectors in the block are zero.
end 
if rat_diag <= stol
   if ~isempty(U_A) && strcmp(reorth,'TWO'), W(:,J) = orthog(W(:,J),U_A,SVTol); end 
   [W(:,J),S_2] = qr(W(:,J),0); S = S_2*S;
   singblk(1) = 1; % Set the singular block indicator to true.
end

% Begin of main iteration loop for the block Lanczos bidiagonalization decomposition.
while (J(blsz) <= blsz*nbls)

   % Matrix A' product with vector(s), (F = A'*W). 
   if interchange
      if ischar(A)
         F = feval(A,W(:,J),m,n,blsz,'F');
      else
         F = A*W(:,J);  
      end
   else
      if ischar(A)
        F = feval(A,W(:,J),m,n,blsz,'T');
      else
        F = (W(:,J)'*A)';
      end
   end
     
   % Count the number of matrix vector products.
   mprod = mprod + blsz;
   
   % One step of the block classical Gram-Schmidt process.
   F = F - V(:,J)*S';

   % Orthogonalize F against all converged right singular vectors V_A.
   if ~isempty(V_A), F = orthog(F,V_A,SVTol); end
   
   % Full Reorthogonalization step. "Short vectors".
   F = orthog(F,V(:,1:J(blsz)),SVTol);

   if J(blsz)+blsz <= blsz*nbls

       % Compute the QR factorization of F.
       [V(:,J+blsz),R] = qr(F,0);
      
       % Check for linearly dependent vectors.
       R_max = max(abs(diag(R))); R_min=min(abs(diag(R)));
       stol = max(SVTol,eps*R_max);
       if R_max > stol
          rat_diag = R_min/R_max;
       else
          rat_diag = 0; % All vectors in the block are numerically zero.
       end 
       if rat_diag <= stol  
         V(:,J+blsz) = orthog(V(:,J+blsz),V(:,1:J(blsz)),SVTol);
         if ~isempty(V_A), V(:,J+blsz) = orthog(V(:,J+blsz),V_A,SVTol); end
         [V(:,J+blsz),R_2] = qr(V(:,J+blsz),0); R = R_2*R;
         singblk(1) = 1; % Set the singular block indicator to true. 
       end 
           
       % Compute block bidiagonal matrix B. 
       if isempty(B) 
         B = [S R'];
       else
         B = [B zeros(J(1)-1,blsz); zeros(blsz,J(1)-1) S R'];
       end 
    
       % Matrix A product with vector(s), (W=A*V).
       if interchange
         if ischar(A)
           W(:,J+blsz) = feval(A,V(:,J+blsz),m,n,blsz,'T');
         else
           W(:,J+blsz) = (V(:,J+blsz)'*A)';
         end
       else
         if ischar(A)
            W(:,J+blsz) = feval(A,V(:,J+blsz),m,n,blsz,'F');
         else
            W(:,J+blsz) = A*V(:,J+blsz);
         end
       end

       % Count the number of matrix vector products.
       mprod = mprod + blsz;

       % One step of the block classical Gram-Schmidt process.
       W(:,J+blsz) =  W(:,J+blsz) - W(:,J)*R';

      % Orthogonalize W against all converged left singular vectors U_A.
       if (~isempty(U_A) && strcmp(reorth,'TWO')) 
           W(:,J+blsz) = orthog(W(:,J+blsz),U_A,SVTol); 
       end
     
      % Full Reorthogonalization step. "Long vectors"
      if ( iter == 1 || strcmp(reorth,'TWO') )
         W(:,J+blsz) = orthog(W(:,J+blsz),W(:,1:J(blsz)),SVTol);
      end
   
      % Compute the QR factorization of W.
      [W(:,J+blsz),S] = qr(W(:,J+blsz),0);
 
      % Check for linearly dependent vectors.
      S_max = max(abs(diag(S))); S_min=min(abs(diag(S)));
      stol = max(SVTol,eps*S_max);
      if S_max > stol
         rat_diag = S_min/S_max;
      else
        rat_diag = 0; % All vectors in the block are zero.
      end 
      if rat_diag <= stol 
         W(:,J+blsz) = orthog(W(:,J+blsz),W(:,1:J(blsz)),SVTol);
         if ~isempty(U_A)  
           W(:,J+blsz) = orthog(W(:,J+blsz),U_A,SVTol); 
         end
        [W(:,J+blsz),S_2] = qr(W(:,J+blsz),0); S = S_2*S;
        singblk(1) = 1; % Set the singular block indicator to true.
      end      

   else
    
    % Add last block to matrix B
     B = [B; zeros(blsz,J(1)-1) S]; 

   end

    % Update iteration count.
    J = J + blsz;

end

function Y = orthog(Y,X,SVTol)
% Orthogonalize vectors Y against vectors X.

if size(X,2) < size(Y,2), dotY = X'*Y; else dotY = (Y'*X)'; end
Y = Y - X*dotY; 
if norm(dotY) > SVTol
   if size(X,2) < size(Y,2), dotY = X'*Y; else dotY = (Y'*X)'; end
   if norm(dotY) > SVTol, Y = Y - X*dotY; end
end

 
%----------------------------------------------------%
% END: BLOCK LANCZOS BIDIAGONALIZATION DECOMPOSITION %
%----------------------------------------------------%


%--------------------------%
% BEGIN: CONVERGENCE TESTS %
%--------------------------%

function [conv,deflate,singval,V_A,U_A,sigma,chsigma,singresdisp,singconv,singblk,Smax,sizint,K,Smin,rhres] = ...
         convtests(computvec,conv,deflate,disps,singval,V_A,U_A,singresdisp,...
                   iter,K,residuals,S_B,singconv,U_B,V_B,B,sigma,...
                   chsigma,nval,tol,SVTol,Bsz,V,W,singblk,Smax,F,blsz,A,sizint,Smin,condnum)
% This function checks the convergence of singular values and singular vectors. 

% James Baglama
% DATE: 11/06/01
 

show = 0;

% Initialization of local variables.
dif  = []; % Place holder for the difference of sets Jr(Jre) and Jrv(Jrev).
Jr   = []; % Place holder for which singular values converged.
Jre  = []; % Place holder for which desired singular values onverged.
Jrv  = []; % Place holder for which singular vectors converged.
Jrev = []; % Place holder for which desired singular vectors converged.
rhres=residuals;
%%%%%%%%%%%%%%%%%
% Jia and Niu refined Harmonic approximations
%%%%%%%%%%%%%%%%%

% Compute Harmonic Values
  % Compute the QR-factorization of F.
global Jia; 
if Jia == 1
  [F,R_F] = qr(F,0); EB = rot90(flipud(eye(Bsz,blsz)))';
  [wB,tt,sB] = svd([B EB*R_F']'); wB = inv(B)*sB; 
  tt = diag(tt);  
  for i=1:Bsz
      wB(:,i) = tt(i)*wB(:,i);
      wB(:,i) = wB(:,i)/norm(wB(:,i));
  end    
  [Y,I] = sort(tt); tt= tt(I); wB = wB(:,I); sB = sB(:,I);
  for i=1:Bsz
      tt(i) = sB(:,i)'*B*wB(:,i);
  end
 
  
  
  TT = [zeros(Bsz) B; B' zeros(Bsz); (EB*R_F')' zeros(blsz,Bsz)]; 
  for i=1:Bsz
   [UU,SS, z] = svd(TT - tt(i)*eye(2*Bsz+blsz,2*Bsz));
   SS = diag(SS);
   [Y,I] = sort(SS); z=z(:,I); sB(:,i) = z(1:Bsz,1); wB(:,i) = z(Bsz+1:2*Bsz,1);
   sB(:,i) = sB(:,i)/norm(sB(:,i)); wB(:,i) = wB(:,i)/norm(wB(:,i));
  end 
  
  for i=1:Bsz
   residuals(1,i) = sqrt(norm(A*V*wB(:,i) - tt(i)*W*sB(:,i))^2 + norm(A'*W*sB(:,i) - tt(i)*V*wB(:,i))^2);
  end
  rhres = residuals;
  S_B = tt;
  V_B = wB;
  U_B = sB;
  if condnum, Smin = min(Smin,S_B(1)); end
end  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Check for convergence of singular values and vectors.
switch sigma
         
   % Smallest singular values.      
   case 'SS'
      
      % Check for convergence of singular vectors.
         Jrv  = find(residuals < SVTol*Smax);
         Jrev = find(Jrv <= K-conv);
      
      % Check for convergence of singular values.
      Jr =   find(residuals < tol*Smax);
      Jre =  find(Jr  <= K-conv);
      
      % Output intermediate results.
      if disps ~= 0 
         dispsing = [singval;S_B(1:K-conv)];
         disperr = [singresdisp';residuals(1:K-conv)'];
         [sortval,JI] = sort(dispsing); dispsing = dispsing(JI); disperr = disperr(JI);
         dispsing = dispsing(1:K); disperr = disperr(1:K);
         disp(sprintf('      Singular Value     Residual       Iteration: %d',iter));
         S = sprintf('%2i %15.5e %15.5e \n',[[1:length(dispsing)]; dispsing';disperr']);
         disp(S); disp(' '); disp(' ');
      end
           
   % Largest singular values.
   case 'LS'
             
      % Check for convergence of singular vectors.
         Jrv = find(residuals < SVTol*Smax);
         Jrev = find(Jrv >= Bsz-K+1+conv);     

      % Check for convergence of singular values.
        Jr =   find(residuals < tol*Smax);
        Jre =  find(Jr  >= Bsz-K+1+conv);

      % Output intermediate results.
      if disps ~= 0
         dispsing = [singval;S_B(Bsz-K+1+conv:Bsz)];
         disperr = [singresdisp';residuals(Bsz-K+1+conv:Bsz)'];
         [sortval,JI] = sort(dispsing); dispsing = dispsing(JI); disperr = disperr(JI);
         dispsing = dispsing(length(dispsing)-K+1:length(dispsing)); 
         disperr = disperr(length(dispsing)-K+1:length(dispsing));
         disp(sprintf('      Singular Value     Residual       Iteration: %d',iter));
         S = sprintf('%2i %15.5e %15.5e \n',[[1:length(dispsing)]; dispsing';disperr']);
         disp(S); disp(' '); disp(' ');
      end
      
% Singular values near nval.
   case 'IS'
       
      % Determine a window where the desired Singular values will occur. 
      [sortval,JI] = sort(abs(S_B-nval));
      S_B = S_B(JI); V_B = V_B(:,JI); U_B = U_B(:,JI);
      residuals = residuals(JI);
     
      % Check for convergence of Singular vectors.
      Jrv  = find(residuals < SVTol*Smax);
      Jrev = find(Jrv <= K-conv);
       
      % Check for convergence of Singular values.
      Jr =   find(residuals < tol*Smax);
      Jre =  find(Jr  <= K-conv);
       
      % Output intermediate results.
      if disps ~= 0 
         % Sort output values.
         [sortval,JI] = sort(S_B(1:K-conv));
         ssingval = S_B(1:K-conv); sresiduals =residuals(1:K-conv);
         ssingval = ssingval(JI); sresiduals = sresiduals(JI);
         dispsing = [singval;ssingval];
         disperr = [singresdisp';sresiduals'];
         [sortval,JI] = sort(dispsing); dispsing = dispsing(JI); disperr = disperr(JI);
         dispsing = dispsing(1:K); disperr = disperr(1:K);
         disp(sprintf('      Ritz           Residual       Iteration: %d',iter));
         S = sprintf('%15.5e %15.5e \n',[dispsing';disperr']);
         disp(S); disp(' '); disp(' ');
      end
      
   % Computing the condition number of A.      
   case 'CN'
      
      % Check for convergence of singular vectors.
      Jrv  = find(residuals < SVTol*Smax);
      Jrevs = find(Jrv == 1); 
      if ~isempty(Jrevs)  
          chsigma=1; 
          sigma='LS';
          sizint = Bsz - blsz - 1; 
      end
      Jrevl = find(Jrv == Bsz); 
      if ~isempty(Jrevl) 
          chsigma=1;  
          sigma='SS'; 
          sizint = Bsz - blsz - 1;
      end
      Jrev = union(Jrevs,Jrevl);
      
      % Check for convergence of singular values.
      Jr =   find(residuals < tol*Smax);
      Jres = find(Jr == 1); 
      Jrel = find(Jr == Bsz); 
      Jre = union(Jres,Jrel); 
            
end % Switch sigma.

% Remove common values in Jre and Jrev. Common values indicate a desired 
% singular triplet has converged and will be deflated.
if ~isempty(Jr(Jre)) && ~isempty(Jrv(Jrev))
   dif = setdiff(Jr(Jre),Jrv(Jrev));
else
   dif = Jr(Jre);
end 

% Determine the number of converged  vectors.
conv = conv + length(Jrev);

% Determine if the requested number of desired singular values have converged.
if conv+length(dif) >= K 
   singval = [singval;S_B(union(Jr(Jre),Jrv(Jrev)))];
      
   % If eigenvectors are requested then compute left and right singular vectors.
   if strcmp(computvec,'T')
      if isempty(V_A)
         V_A = V*V_B(:,union(Jr(Jre),Jrv(Jrev)));
         U_A = W*U_B(:,union(Jr(Jre),Jrv(Jrev)));
	  else
		  if show
			size(V_A)
			Jr(Jre)
			Jrv(Jrev)
			union(Jr(Jre),Jrv(Jrev))
			size(V_B(:,union(Jr(Jre),Jrv(Jrev))))
			size(U_B(:,union(Jr(Jre),Jrv(Jrev))))
			size(V)
			size(W)
		  end
         V_A = [V_A,V*V_B(:,union(Jr(Jre),Jrv(Jrev)))];
         U_A = [U_A,W*U_B(:,union(Jr(Jre),Jrv(Jrev)))];
      end   
   end   
   
   % Set convergence to true and exit.
   singconv = 'T'; return;
   
end  

% Compute converged singular vectors so they can be deflated.    
if length(Jrv) > 0
   singval = [singval;S_B(Jrv)];
   singresdisp = [singresdisp,residuals(Jrv)];
   if isempty(V_A)
      V_A = V*V_B(:,Jrv);
      U_A = W*U_B(:,Jrv);
   else   
      V_A = [V_A,V*V_B(:,Jrv)];
      U_A = [U_A,W*U_B(:,Jrv)];
   end   
   singblk = 0;     
 
   % Check for convergence of singular vectors that do not occur in
   % the desired part of the spectrum. The end points of the 
   % dampening interval(s) will be reset in the main iteraion loop.
   if abs(length(Jrev)-length(Jrv)) ~= 0, deflate = 1; end
end


%------------------------%
% END: CONVERGENCE TESTS %
%------------------------%

%-----------------------------------------------------------%  
% BEGIN: DETERMINE INTERVALS, LEJA POINTS, AND APPLY SHIFTS %
%-----------------------------------------------------------%

function [fLejapts,lcandpts,leftendpt,leftintendpt,lprd,leftLejapt,lindex,maxdpol,nbls,norlpol,...
         nval,rcandpts,rightendpt,rightintendpt,rprd,rightLejapt,rindex,V,flcount] =... 
         applyshifts(blsz,endpt,F,fLejapts,K,lcandpts,leftendpt,leftintendpt,...
                     lprd,leftLejapt,lindex,maxdpol,nbls,norlpol,rcandpts,...
                     rightendpt,rightintendpt,ritz,V_B,rprd,rightLejapt,rindex,...
                     zertyp,sigma,nval,singblk,sizint,Bsz,B,V,flcount,sqrteps,residuals)
% This function determines the endpoints of the dampening intervals, zeros (weighted Leja 
% points or mapped Leja points) and applies the zeros as shifts. Leja points are computed in
% for dampening eigenvalues of A'*A. The shifts are applied indirectly to A'*A. 

% James Baglama
% DATE: 11/06/01

% Determine intervals for dampening.
switch sigma
       
   % Searching for the smallest singular values.
   case 'SS' 
      if isempty(rightendpt)
         rightintendpt  = ritz(Bsz-sizint); rightendpt = ritz(Bsz);
         nextritz = ritz(Bsz-sizint-1);
      else
         if ~singblk(1) % If a singular block is detected, do not modify end points. 
            if strcmp(endpt,'MON')
               rightintendpt = min(ritz(Bsz-sizint),rightintendpt);
            else
               rightintendpt  = ritz(Bsz-sizint);
            end   
              rightendpt = max(ritz(Bsz),rightendpt);
         end   
      end
      % Find the closest ritz value. 
      nextritz = ritz(K); nextres = residuals(K);
      
      
      % Choose a value to scale the Leja polynomial to avoid underflow/overflow.
      if isempty(fLejapts), norlpol = ritz(1); end   
      
   % Searching  for the largest singular values.
   case 'LS' 
      if isempty(leftendpt) 
         leftendpt = ritz(1);leftintendpt = ritz(1+sizint);
      else
         if ~singblk(1) % If a singular block is detected, do not modify end points.
            if strcmp(endpt,'MON')
               leftintendpt = max(ritz(1+sizint),leftintendpt);
            else
               leftintendpt = ritz(1+sizint);
            end   
            leftendpt = min(ritz(1),leftendpt);
         end  
      end
      
      % Find the closest ritz value.
      [Y,I] = min(abs(ritz- leftintendpt)); 
      nextritz = ritz(I); nextres = residuals(I);
      
      % Choose a value to scale the Leja polynomial to avoid underflow/overflow.
      if isempty(fLejapts), norlpol = ritz(length(ritz)); end

   % Searching for interior singular values near nval.
   case 'IS'
     
      % Set norlpol.
      norlpol = 0;
                            
      % Check the minimum eigenvalues of T = B'*B - nval^2*I.
      [rmin,ri] = min(abs(ritz-nval^2));
      
      % If minimum eigenvalue < sqrt(eps) adjust nval to avoid a singular matrix T.
      if (rmin < sqrteps)
                          
         % If the matrix T is singular adjust nval to be the midpoint between nval 
         % and the next closest singular value that it is not equal to. 
         if ((ri > 1) && (ri < Bsz))
             sing = sqrt(ritz);
             adjust = min(abs(sing(ri-1)-sing(ri))/2,abs(sing(ri+1)-sing(ri))/2);
         end
         if ri == 1,   adjust = abs(sing(ri+1)-sing(ri))/2; end 
         if ri == Bsz, adjust = abs(sing(ri-1)-sing(ri))/2; end
         if adjust < sqrteps, adjust = 2*sqrteps; end 
         nval = nval + adjust;  
      end   
      
      % Shift the spectrum of T by nval^2 for computing Harmonic Ritz
      % values.
      ritz = ritz - nval^2; T = B'*B - nval^2*speye(Bsz);
      
      % Compute the Harmonic Ritz values.
      S = B(Bsz-blsz+1:Bsz,Bsz-blsz+1:Bsz); % Last diagonal block of B.
      R  = sparse(triu(qr((F*S),0))); R = R(1:blsz,1:blsz);
      ER = sparse(R*rot90(flipud(speye(Bsz,blsz))));
      Hritz = eig(full(T + T\(ER)'*ER)); Hritz = sort(real(Hritz));
       
      % Set extreme endpoints.
      if isempty(leftendpt)
         leftendpt = ritz(1); rightendpt = ritz(Bsz);
      else
         leftendpt = min(leftendpt-nval^2,ritz(1)); 
         rightendpt = max(rightendpt-nval^2,ritz(Bsz));
      end   
     
      % Determine interior endpoints.
      kn = 1 + sizint; 
      kp = length(Hritz)-sizint;
           
      while ((kn < length(Hritz)) && (kp > 1))
                  
         % Check to see if the "left" side (negative) interval is acceptable.
         if isempty(max(find(Hritz<0)))
             leftintendpt = [];
         else
             if ((kn < (max(find(Hritz<0)) - round(K/2))) && (Hritz(kn) > leftendpt))
                if strcmp(endpt,'FLT'), leftintendpt = Hritz(kn); end 
                if strcmp(endpt,'MON')
                   if isempty(leftintendpt)
                      leftintendpt = Hritz(kn); 
                   else
                      leftintendpt = max(leftintendpt,Hritz(kn)); 
                   end   
                end   
             else
                leftintendpt = []; 
             end
         end    
         
         % Check to see if the "right" side (positive) interval is acceptable.
         if isempty(min(find(Hritz > 0)))
            rightintendpt = [];
         else 
            if ((kp >(min(find(Hritz > 0)) + round(K/2))) && (Hritz(kp) < rightendpt))
               if strcmp(endpt,'FLT'), rightintendpt = Hritz(kp); end
               if strcmp(endpt,'MON')
                  if isempty(rightintendpt)
                     rightintendpt = Hritz(kp); 
                  else
                     rightintendpt = min(rightintendpt,Hritz(kp)); 
                  end 
               end   
            else
               rightintendpt = []; 
            end
         end    
        
         if (~(isempty(leftintendpt)) && ~(isempty(rightintendpt))), break; end
         if isempty(leftintendpt), kn = kn + 1; end
         if isempty(rightintendpt), kp = kp-1; end
         
         
         % Both intervals are unacceptable. Adjust locations and re-compute.
         %if ((isempty(leftintendpt)) && (isempty(rightintendpt)))
         %   kn = kn + 1; kp = kp - 1; 
        % else
        %    break;
        % end
         
      end % While loop.
  
      % No interior endpoints, hence dampening intervals, can not be picked. Increase the 
      % number of blocks and restart with BLSZ number of Ritz vectors that are closest
      % to the desired value NVAL. If the matrix T has a singular block, then reset the
      % interior endpoints.
      if ((isempty(leftintendpt)) && (isempty(rightintendpt)) || singblk(1)) 
         lcandpts = []; lindex = []; lprd = []; rcandpts = []; rindex = []; rprd = []; fLejapts = [];
         [sortval,JI] = sort(abs(ritz));
         if singblk(1) && blsz > 1
            % Singular block in B and V needs to be modified.
            leftintendpt = []; rightintendpt = [];
         end 
         V = V*V_B(:,JI(1:blsz)); nbls = nbls + 1; 
         warning(['Increasing NBLS to ',num2str(nbls)]);
         return;
      end 
      
     % Shift endpoints to the interior part of the spectrum for singular values.
     leftendpt = leftendpt + nval^2;
     rightendpt = rightendpt + nval^2;
     if ~isempty(leftintendpt),  leftintendpt  = leftintendpt + nval^2; end
     if ~isempty(rightintendpt), rightintendpt = rightintendpt + nval^2; end 
     
     %if isempty(leftintendpt),
     %   S = sprintf('%15.5e %15.5e %15.5e %15.5e \n',[leftendpt 00.00 rightintendpt rightendpt].^(0.5));
     %   disp(S);
     %end
     %if isempty(rightintendpt)
     %   S = sprintf('%15.5e %15.5e %15.5e %15.5e \n',[leftendpt leftintendpt 00.00 rightendpt].^(0.5));
    %disp(S);
    % end
    % if (~(isempty(leftintendpt)) && ~(isempty(rightintendpt)))
    %     S = sprintf('%15.5e %15.5e %15.5e %15.5e \n',[leftendpt leftintendpt rightintendpt rightendpt].^(0.5));
    %    disp(S);
    % end    
     
     
    % Computing condition number. 
    case 'CN'     
     
         middle = nval;
         [Y,I] = min(abs(ritz-middle));  
        if isempty(rightendpt)
            if I-sizint >= 2, rightintendpt  = ritz(I-sizint); else  rightintendpt  = ritz(2); end
            if I+sizint < Bsz, rightendpt = ritz(I+sizint); else rightendpt = ritz(Bsz-1); end         
        else
         if ~singblk(1) % If a singular block is detected, do not modify end points. 
            if strcmp(endpt,'MON')
               if I-sizint >= 2, 
                  rightintendpt = min(ritz(I-sizint),rightintendpt);
               else
                  rightintendpt = min(ritz(2),rightintendpt);
               end 
               if I+sizint < Bsz
                  rightendpt = max(ritz(I+sizint),rightendpt);
               else
                  rightendpt = max(ritz(Bsz-1),rightendpt); 
               end   
            else
               if I-sizint >= 2, rightintendpt  = ritz(I-sizint); else  rightintendpt  = ritz(2); end
               if I+sizint < Bsz, rightendpt = ritz(I+sizint); else rightendpt = ritz(Bsz-1); end
            end   
    
         end   
        end
        
        
        % Set norlpol.
        norlpol = 0;
            
end % Switch sigma.


%%%%%%

       
%%%%%%%%%        

% Determine shifts. Begin of switch for zertyp.
switch zertyp
    
   % Compute fast Leja points on [-2,2] and map to the dampening interval. 
   case 'ML'
      if flcount+nbls > maxdpol
         flcount = 0; maxdpol = length(fLejapts); 
      end
      if flcount == length(fLejapts)
         [fLejapts,rcandpts,rindex,rprd] = fastleja(fLejapts,nbls,rcandpts,rindex,rprd);
      end
      pshifts = fLejapts(flcount+1:nbls+flcount);
      flcount = flcount + length(pshifts);
      if strcmp(sigma,'SS')   
         pshifts = (rightendpt-rightintendpt)/4*(pshifts-2) + rightendpt;
      else
         pshifts = (leftintendpt-leftendpt)/4*(pshifts-2) + leftintendpt;
      end    
                    
   % Compute weighted fast Leja points.     
   case 'WL' 
      if length(fLejapts) + nbls > maxdpol
         % Reset values.
         lcandpts = []; lindex = []; lprd = []; rcandpts = []; rindex = []; rprd = []; fLejapts = [];
         leftLejapt = []; rightLejapt = [];
         if strcmp(sigma,'SS'), norlpol = ritz(1); end
         if strcmp(sigma,'LS'), norlpol = ritz(length(ritz)); end
         if strcmp(sigma,'IS') || strcmp(sigma,'CN'), norlpol = 0; end         
      end       
      
      
      [fLejapts,lcandpts,leftendpt,leftintendpt,lprd,leftLejapt,lindex,pshifts,...
      rcandpts,rightendpt,rightintendpt,rprd,rightLejapt,rindex] = ...
      fastlejawght(fLejapts,lcandpts,leftendpt,leftintendpt,lprd,...
                   leftLejapt,lindex,norlpol,nbls,rcandpts,rightendpt,...
                   rightintendpt,rprd,rightLejapt,rindex);
                 
      % Check for early termination and reset values and re-compute fast Leja points.
      if (length(pshifts) ~= nbls)
         % Reset values, and try to compute Leja points from the beginning.
         lcandpts = []; lindex = []; lprd = []; rcandpts = []; rindex = []; rprd = []; fLejapts = [];
         leftLejapt = []; rightLejapt = [];
         if strcmp(sigma,'SS'), norlpol = ritz(1); end
         if strcmp(sigma,'LS'), norlpol = ritz(length(ritz)); end
         if strcmp(sigma,'IS') || strcmp(sigma,'CN'), norlpol = 0; end  
         [fLejapts,lcandpts,leftendpt,leftintendpt,lprd,leftLejapt,lindex,pshifts,...
         rcandpts,rightendpt,rightintendpt,rprd,rightLejapt,rindex] = ...
         fastlejawght(fLejapts,lcandpts,leftendpt,leftintendpt,lprd,...
                      leftLejapt,lindex,norlpol,nbls,rcandpts,rightendpt,...
                      rightintendpt,rprd,rightLejapt,rindex);
         % Reset values, if an error has occurred and return using app. singular vectors.
         if (length(pshifts) ~= nbls)
            lcandpts = []; lindex = []; lprd = []; rcandpts = []; rindex = []; rprd = []; fLejapts = [];
            leftLejapt = []; rightLejapt = [];
            [sortval,JI] = sort(abs(ritz));
            V = V*V_B(:,JI(1:blsz));
            return;
         end   
      end 
    % Check the relative gap. If a s hift is too close to approximate
    % singular value then remove shift and compute a another Leja shift.
    %relgap = min(abs( ((nextritz-nextres) - pshifts)./nextritz));
      
            
end % Switch zertyp.





% Apply nbls-1 shifts.
for J = nbls:-1:2
   Jblsz = J*blsz; Jmblsz = Jblsz-blsz;
   B = B(1:Jblsz,1:Jblsz); 
   [B,Q_r,Q_l] = bulgechasing(B,pshifts(nbls-J+1),blsz,Jblsz);
   V = V(:,1:Jblsz)*Q_r;
   F = V(:,Jmblsz+1:Jblsz)*B(Jblsz-2*blsz+1:Jmblsz,Jmblsz+1:Jblsz)'...     
       + F*Q_l(Jmblsz+1:Jblsz,Jblsz-2*blsz+1:Jmblsz);
end  
   
% Apply the last shift.
B = B(1:blsz,1:blsz); V = F*B + V(:,1:blsz)*(B'*B - pshifts(nbls)*eye(blsz) );

%---------------------------------------------------------%  
% END: DETERMINE INTERVALS, LEJA POINTS, AND APPLY SHIFTS %
%---------------------------------------------------------%

%-------------------------------------------------------------------------------%
% BEGIN: FAST LEJA POINT ROUTINE WITH WEIGHT FUNCTION FOR ONE OR TWO INTERVALS. %
%-------------------------------------------------------------------------------%

function [fLejapts,lcandpts,leftendpt,leftintendpt,lprd,leftLejapt,lindex,pshifts,...
         rcandpts,rightendpt,rightintendpt,rprd,rightLejapt,rindex] = ...
         fastlejawght(fLejapts,lcandpts,leftendpt,leftintendpt,lprd,...
                      leftLejapt,lindex,norlpol,numshfts,rcandpts,...
                      rightendpt,rightintendpt,rprd,rightLejapt,rindex)
% This function computes the fast Leja points with a weight function for one or two intervals. 
%
% References:
% 1.) "Fast Leja Points", J. Baglama, D. Calvetti, and L. Reichel, ETNA (1998) Volume 7.
% 2.) "Iterative Methods for the Computation of a Few Eigenvalues of a Large Symmetric Matrix",
%      J. Baglama, D. Calvetti, and L. Reichel, BIT, 36 (1996), pp. 400-421.

% James Baglama 
% DATE: 11/06/01

% Number of previously generated fast Leja points.
nfLejapts = length(fLejapts); 
    
% Initialize arrays.
pshifts = []; % Output array that holds the shifts to be applied. 
IL = [];      % Place holder for finding points in the left intervals.
IR = [];      % Place holder for finding points in the right intervals.

% Checking for errors in input argument(s). 
if (numshfts < 1) 
   error('Number of shifts is (<= 0) in fast Leja with weight function.');
end
if ((isempty(leftintendpt)) & (isempty(rightintendpt)))
   error('Empty endpoints in fast Leja with weight function.');
end

% Initialize the arrays fLejapts, lcandpts, and rcandpts.
if ((isempty(rcandpts)) & (~isempty(rightintendpt)))
   nfLejapts = nfLejapts+1;
   fLejapts(nfLejapts) = rightendpt; rightLejapt = nfLejapts; 
   rcandpts(1) = (rightintendpt+fLejapts(nfLejapts))/2;
   rprd(1) = prod((rcandpts(1)-fLejapts)./(norlpol-fLejapts));
   rindex(1,1) = 0; rindex(1,2) = nfLejapts;
   numshfts = numshfts-1; pshifts = fLejapts(nfLejapts);
end   
if ((isempty(lcandpts)) & (~isempty(leftintendpt)))
   nfLejapts = nfLejapts+1;
   fLejapts(nfLejapts) = leftendpt; leftLejapt = nfLejapts;   
   lcandpts(1) = (leftintendpt+fLejapts(nfLejapts))/2;     
   lprd(1) = prod((lcandpts(1)-fLejapts)./(norlpol-fLejapts));
   lindex(1,1) = nfLejapts; lindex(1,2) = 0;
   numshfts = numshfts-1; pshifts = [pshifts,fLejapts(nfLejapts)];
end

% Set products of the end points to zero.
rightprd = 0; leftprd = 0;

% Find closest Leja points in each interval to the interior end points.
sizlcpts = length(lcandpts); sizrcpts = length(rcandpts);    
if ((sizlcpts > 1) & (~isempty(leftintendpt)))
   IL = find(fLejapts < leftintendpt);[YL,KL] = max(fLejapts(IL));      
   if ~isempty(IL(KL))
      lcandpts(1) = (leftintendpt+fLejapts(IL(KL)))/2;     
      lprd(1) = prod((lcandpts(1)-fLejapts)./(norlpol-fLejapts));
      lindex(1,1) = IL(KL); lindex(1,2) = 0;
   else
      lprd(1) == 0;
   end
   leftprd = prod((leftendpt-fLejapts)./(norlpol-fLejapts)); 
end
if ((sizrcpts > 1) & (~isempty(rightintendpt)))
   IR = find(fLejapts > rightintendpt);[YR,KR] = min(fLejapts(IR));      
   if ~isempty(IR(KR))
      rcandpts(1) = (rightintendpt+fLejapts(IR(KR)))/2;     
      rprd(1) = prod((rcandpts(1)-fLejapts)./(norlpol-fLejapts));
      rindex(1,1) = 0; rindex(1,2) = IR(KR);
   else
      rprd(1) = 0;
   end
   rightprd = prod((rightendpt-fLejapts)./(norlpol-fLejapts));
end

% Initialize values.
JL = [];        % Index of acceptable candidate points in the left interval.
JR = [];        % Index of acceptable candidate points in the right interval.
maxprdJL = -1;  % Maximum value of polynomial on left interval.
maxprdJR = -1;  % Maximum value of polynomial on right interval.
maxJL = -1;     % Index of maximum value of polynomial on left interval.
maxJR = -1;     % Index of maximum value of polynomial on right interval.
  
% Search for Leja points. 
for k = 1+nfLejapts:numshfts+nfLejapts
   
   if ~isempty(leftintendpt)
      JL = find(lcandpts > leftendpt & lcandpts < leftintendpt);
      [maxprdJL,maxJL] = max(abs((lcandpts(JL)-leftintendpt).*lprd(JL)));
      maxJL = JL(maxJL); leftprd = abs((leftendpt-leftintendpt).*leftprd);
      if isempty(maxprdJL),maxprdJL = leftprd; maxJL = 0;
      elseif(leftprd > maxprdJL), maxprdJL = leftprd; maxJL = 0; end
   end
      
   if ~isempty(rightintendpt)
      JR = find(rcandpts > rightintendpt & rcandpts < rightendpt);
      [maxprdJR,maxJR] = max(abs((rcandpts(JR)-rightintendpt).*rprd(JR)));
      maxJR = JR(maxJR); rightprd = abs((rightendpt-rightintendpt).*rightprd);
      if isempty(maxprdJR),maxprdJR = rightprd; maxJR = 0;
      elseif rightprd > maxprdJR , maxprdJR = rightprd; maxJR = 0; end
   end
   
   % Check for overflow or underflow.
   if ((isinf([maxprdJR,maxprdJL])) | (isnan([maxprdJR,maxprdJL])))
      lcandpts = []; rcandpts = []; fLejapts = []; lindex = []; rindex = []; lprd = []; rprd = [];
      rightLejapt = []; leftLejapt = []; pshifts = []; return; 
   end    
    
   % Case 1. Right end point is the next fast Leja point.
   if ((maxprdJR >= maxprdJL) & (maxJR == 0))
      fLejapts(k) = rightendpt; sizrcpts = sizrcpts+1;
      if isempty(IR)
         rcandpts(sizrcpts) = (rightendpt + rightintendpt)/2;
         rindex(sizrcpts,1) = 0; rindex(sizrcpts,2) = k;
      else   
         rcandpts(sizrcpts) = (rightendpt + fLejapts(rightLejapt))/2;
         rindex(sizrcpts,1) = rightLejapt; rindex(sizrcpts,2) = k;
      end   
      rightLejapt = k; rightprd = 0; 
      rprd(sizrcpts) = prod((rcandpts(sizrcpts)-fLejapts(1:k-1))./(norlpol-fLejapts(1:k-1)));
   end   
   
   % Case 2. Left end point is the next fast Leja point.
   if ((maxprdJL > maxprdJR) & (maxJL == 0))
      fLejapts(k) = leftendpt; sizlcpts = sizlcpts+1;
      if isempty(IL)
         lcandpts(sizlcpts) = (leftendpt + leftintendpt)/2;
         lindex(sizlcpts,1) = k; lindex(sizlcpts,2) = 0;
      else   
         lcandpts(sizlcpts) = (leftendpt + fLejapts(leftLejapt))/2;
         lindex(sizlcpts,1) = k; lindex(sizlcpts,2) = leftLejapt;
      end
      leftLejapt = k; leftprd = 0;
      lprd(sizlcpts) = prod((lcandpts(sizlcpts)-fLejapts(1:k-1))./(norlpol-fLejapts(1:k-1)));
   end   
   
   % Case 3. Leja point picked in Left interval.
   if ((maxprdJL > maxprdJR) & (maxJL ~= 0))
      fLejapts(k) = lcandpts(maxJL); sizlcpts = sizlcpts+1; 
      lindex(sizlcpts,1) = k; lindex(sizlcpts,2) = lindex(maxJL,2); lindex(maxJL,2) = k;
      if (lindex(sizlcpts,2) == 0)
         lcandpts(sizlcpts) = (fLejapts(lindex(sizlcpts,1))+leftintendpt)/2;
      else 
         lcandpts(sizlcpts) = (fLejapts(lindex(sizlcpts,1))+fLejapts(lindex(sizlcpts,2)))/2;
      end
      lcandpts(maxJL) = (fLejapts(lindex(maxJL,1))+fLejapts(lindex(maxJL,2)))/2;
      lprd(maxJL) = prod((lcandpts(maxJL)-fLejapts(1:k-1))./(norlpol-fLejapts(1:k-1)));
      lprd(sizlcpts) = prod((lcandpts(sizlcpts)-fLejapts(1:k-1))./(norlpol-fLejapts(1:k-1)));
   end       
      
   % Case 4. Leja point picked in right interval. 
   if ((maxprdJR >= maxprdJL) & (maxJR ~= 0))
      fLejapts(k) = rcandpts(maxJR); sizrcpts = sizrcpts+1; 
      rindex(sizrcpts,1) = k; rindex(sizrcpts,2) = rindex(maxJR,2); rindex(maxJR,2) = k;
      if (rindex(maxJR,1) == 0)
         rcandpts(maxJR) = (rightintendpt+fLejapts(rindex(maxJR,2)))/2;
      else 
         rcandpts(maxJR) = (fLejapts(rindex(maxJR,1))+fLejapts(rindex(maxJR,2)))/2;
      end
      rcandpts(sizrcpts) = (fLejapts(rindex(sizrcpts,1))+fLejapts(rindex(sizrcpts,2)))/2;
      rprd(maxJR) = prod((rcandpts(maxJR)-fLejapts(1:k-1))./(norlpol-fLejapts(1:k-1)));
      rprd(sizrcpts) = prod((rcandpts(sizrcpts)-fLejapts(1:k-1))./(norlpol-fLejapts(1:k-1)));
   end 
   rprd = rprd.*(rcandpts-fLejapts(k))/(norlpol-fLejapts(k));
   lprd = lprd.*(lcandpts-fLejapts(k))/(norlpol-fLejapts(k));
   if rightprd ~= 0,  rightprd = rightprd.*(rightendpt - fLejapts(k))/(norlpol-fLejapts(k)); end
   if leftprd ~= 0,    leftprd =  leftprd.*(leftendpt - fLejapts(k))/(norlpol-fLejapts(k)); end

end % For loop.

pshifts = [pshifts,fLejapts(nfLejapts+1:nfLejapts+numshfts)];

%-----------------------------------------------------%
% END: FAST LEJA POINT ROUTINE WITH WEIGHT FUNCTION %
%-----------------------------------------------------%

%-------------------------------------------%
% BEGIN: FAST LEJA POINT ROUTINE FOR [-2,2] %
%-------------------------------------------%

function [fLejapts,rcandpts,rindex,rprd] = fastleja(fLejapts,numshfts,rcandpts,rindex,rprd,shifts)
% This function is made to be used in irbleigs.m. This function computes the
% fast Leja points on the interval [-2,2]. 
%
% References:
% "Fast Leja Points", J. Baglama, D. Calvetti, and L. Reichel, ETNA (1998) Volume 7.

% James Baglama
% 10/18/01

% Number of previously generated fast Leja points.
nfLejapts = length(fLejapts); 
    
% Checking for errors in input argument(s). 
if (numshfts < 1) 
   error('Number of shifts is 0 or (< 0) in fast Leja routine.');
end
   
% Initialize the array fLejapts.
if (nfLejapts < 2)
   fLejapts(1) = 2;
   if ((nfLejapts == 0) & (numshfts == 1)); return; end 
   fLejapts(2) = -2; rcandpts(1) = (fLejapts(1)+fLejapts(2))/2;
   rprd(1) = prod(rcandpts(1)-fLejapts); rindex(1,1) = 2; rindex(1,2) = 1;
   if ((nfLejapts == 0) & (numshfts == 2)); return; end
   if ((nfLejapts == 1) & (numshfts == 1)); return; end
   nfLejapts = 2; numshfts = numshfts-2;
end    
    
% Main loop to calculate the fast Leja points.
for k = 1+nfLejapts:numshfts+nfLejapts
   [maxval,maxk]  = max(abs(rprd));   
   fLejapts(k)    = rcandpts(maxk);
   rindex(k-1,1)  = k; rindex(k-1,2) = rindex(maxk,2); rindex(maxk,2) = k;
   rcandpts(maxk) = (fLejapts(rindex(maxk,1))+fLejapts(rindex(maxk,2)))/2;
   rcandpts(k-1)  = (fLejapts(rindex(k-1,1))+fLejapts(rindex(k-1,2)))/2;
   rprd(maxk)     = prod(rcandpts(maxk)-fLejapts(1:k-1));
   rprd(k-1)      = prod(rcandpts(k-1)-fLejapts(1:k-1));
   rprd           = rprd.*(rcandpts-fLejapts(k));
end

%--------------------------------------------%
% END: FAST LEJA POINT ROUTINE ONE INTERVAL. %
%--------------------------------------------%

%------------------------------------%
% BEGIN: BLOCK BULGECHASING ROUTINE. %
%------------------------------------%
function [B,Q_r,Q_l] = bulgechasing(B,z,blsz,Bsz)
% Bulgechasing Algorithm
%
% Input:   Block bidiagonal matrix B, Q, and shift z.
% Output:  Updated block bidiagonal matrix B and orthogonal matrices Q.

% James Baglama
% 3/09/03
%

% Create the first blsz rows of B'*B and implicitly apply the shift z.
Y = B(1:blsz,1:2*blsz)'*B(1:blsz,1:blsz); for i=1:blsz, Y(i,i) = Y(i,i) - z; end

% Householder transformations (via MATLAB routine qr) are used to do the bulgechasing. 
Q_r = eye(Bsz); Q_l = Q_r; [Q,R] = qr(Y); 

for i = 1:blsz:Bsz-1

    % Set indices.
    m = (i-1)+2*blsz; m2 = m + blsz; 
    if m >= Bsz, m = Bsz; m2 = Bsz; end 
     
    % Update B from the left and Q_r.
    B(:,i:m) = B(:,i:m)*Q; Q_r(:,i:m) = Q_r(:,i:m)*Q;

    % Get the next column to zero out and create the householder matrix.
    [Q,R] = qr(B(i:m,i:(i-1)+blsz));  
    
    % Update B from the right and Q_l.  
    B(i:m,:) = Q'*B(i:m,:); 
    Q_l(:,i:m) = Q_l(:,i:m)*Q;     

   % Get the next row to zero out and create the householder matrix.
   if i < Bsz - blsz
      [Q,R] = qr(B(i:(i-1)+blsz,blsz+i:m2)');
   end
end
%----------------------------------%
% END: BLOCK BULGECHASING ROUTINE. %
%----------------------------------%
