function varargout = irlba_mod(varargin)
%
% THIS CODE IS MODIFIED TO SEARCH FOR ONLY THE LARGEST SINGULAR TRIPLETS.  
% AFTER FIRST CALL, OUTPUTS THE NEEDED VALUES TO RESTART AND SEARCH FOR 
% ADDITIONAL VALUES.
%
% FIRST CALL:
% Set:     opts.ext = 1;   To not extend (i.e. first call).
%          opts.k = 10;    Number of desired singular triplets.
%          opts.m_b = 20;  About 10 more than than k.
% Output:  
%          U is an orthonormal m x k matrix of "left" singular vectors
%          V is an orthonormal n x k matrix of "right" singular vectors
%       FLAG  see below
%          F is n x 1 matrix and is used for restarting the system
%          B is a k x K+1 matrix and is used for restarting the system
%
%  Your call to the routine should be: 
%  [U S V FLAG F B] = irlba_mod(A,opts);
%
%  AFTER THE FIRST CALL:
%  Set:
%      opts.ext = 0; to extend the system;
%      opts.k = 20;  (This number is the total number of singular values.
%                     This number includes the previous k plus additional 
%                     singular values.)
%      opts.m_b = 30; About 10 more than than k. 
%      opts.v = V; outputed "right" singular vectors from previous call
%      opts.u = U; outputed "left" singular vectors from previous call
%      opts.F = F; outputed F from previous call
%      opts.B = B; outputed B from previous call
%
%  Your call to the routine should be: 
%  [U S V FLAG F B] = irlba_mod(A,opts);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% IRLBA will find a few largest singular values and singular vectors of A 
% such that A*V = U*S, where V is an orthonormal n x k matrix of "right" singular 
% vectors, U is an orthonormal m x k matrix of "left" singular vectors, and S is 
% a k x k diagonal matrix of singular values. 
%
% [U,S,V,FLAG, F, B] = IRLBA(A,OPTIONS) 
% [U,S,V,FLAG, F, B] = IRLBA('Afunc',M,N,OPTIONS) 
%
% The first input argument into IRLBA can be a numeric matrix A or an M-file 
% ('Afunc'). If the M x N matrix A is a filename then Y = Afunc(X,M,N,'transpose'). 
% If transpose = 'F', then X is an (N x 1) matrix and Y = A*X. If transpose = 'T', 
% then X is an (M x 1) matrix and Y = A'*X.
% 
% OUTPUT:
% ---------------
%
%       [U,S,V, FLAG,F,B] = IRLBA(A)
%       S is a diagonal matrix that contains the desired singular values in descending
%       order along the diagonal, the matrix V contains the corresponding "right" singular
%       vectors, and U contains the corresponding "left" singular vectors such that 
%       that A*V = U*S, V'*V = I, and U'*U = I.  A two dimensional array FLAG 
%       that reports if the algorithm converges and the number of matrix vector 
%       products. If FLAG(1) = 0 then this implies normal return: all desired 
%       singular values have converged. If FLAG(1) = 1 then the maximum number 
%       of iterations have been reached before all desired values converged. 
%       FLAG(2) contains the number of matrix vector products (A*X and A'*X) 
%       used by the code. If the maximum number of iterations are reached 
%       (FLAG(1) = 1), then the matrices U, V, and S contain any singular pairs
%       that have converged plus the last singular pair approximation for the 
%       pairs that have not converged. F is n x 1 matrix and is used for restarting the system
%          B is a k x K+1 matrix and is used for restarting the system
%
% INPUT OPTIONS:
% --------------
%                                   
%       ... = IRLBA(A,OPTS)
%       OPTS is a structure containing input parameters. The input parameters can
%       be given in any order. The structure OPTS may contain some or all of the 
%       following input parameters. The string for the input parameters can contain
%       upper or lower case characters. 
%       
%  INPUT PARAMETER      DESCRIPTION    
%
%  OPTS.K            Number of desired singular triplets.             
%                    DEFAULT VALUE  K = 6
%
%  OPTS.MAXIT        Maximum number of iterations, i.e. maximum number of restarts.                           
%                    DEFAULT VALUE  MAXIT = 1000                      
%
%  OPTS.M_B          Size of the Lanczos bidiagonal matrix. The program may increase
%                    M_B to ensure certain requirements are satisfied. A warning message
%                    will be displayed if M_B increases.                          
%                    DEFAULT VALUE    M_B = 20
%
%  DATE:  4/29/13
%
%  VER:  1.0
%  AUTHORS:
%  James Baglama     University of Rhode Island  E-mail: jbaglama@math.uri.edu
%  Research supported by NSF grant DMS-0311786.
%
%  Lothar Reichel    Kent State University       E-mail: reichel@mcs.kent.edu

% REFERENCES:
% 1.) "Augmented Implicitly Restarted Lanczos Bidiagonalization Methods", 
%      J. Baglama and L. Reichel, SIAM J. Sci. Comput. in press 2005.
     

%----------------------------%
% BEGIN: PARSE INPUT VALUES. %
%----------------------------%

% No input arguments, return help.
if nargin == 0, help irlba, return, end

% Matrix A is stored in varargin{1}. Check type (numeric or character) and dimensions.
if (isstruct(varargin{1})), error('A must be a matrix.'), end
if ischar(varargin{1})
   if nargin == 1, error('Need dimension M for matrix A).'), end  
   m = varargin{2};
   if ~isnumeric(m) || length(m) ~= 1 
      error('Second argument M must be a numeric value.'); 
   end
   if nargin == 2, error('Need dimension N for matrix A).'); end  
   n = varargin{3}; 
   if ~isnumeric(n) || length(n) ~= 1 
      error('Third argument N must be a numeric value.'); 
   end
else
   [m,n] = size(varargin{1});
end

% Set all input options to default values.
adjust = 3; K = 6; maxit = 1000; m_b = 20; sigma = 'LS';  tol = 1d-6;  
aug ='RITZ'; reorth='ONE'; V=[]; EXT = 1;

% Get input options from the structure.
if nargin > 1 + 2*ischar(varargin{1})
   options = varargin{2+2*ischar(varargin{1}):nargin};
   names = fieldnames(options);
   I = strmatch('K',upper(names),'exact');
   if  ~isempty(I), K = getfield(options,names{I}); end
   I = strmatch('MAXIT',upper(names),'exact');
   if ~isempty(I), maxit = getfield(options,names{I}); end
   I = strmatch('M_B',upper(names),'exact');
   if ~isempty(I), m_b = getfield(options,names{I}); end
   I = strmatch('TOL',upper(names),'exact');
   if ~isempty(I), tol = getfield(options,names{I}); end
   %%%%%%%NEW- for resarting---  %%%%%%%%%%
   I = strmatch('F',upper(names),'exact');
   if ~isempty(I), F = getfield(options,names{I}); end
   I = strmatch('B',upper(names),'exact');
   if ~isempty(I), B = getfield(options,names{I}); end
   I = strmatch('EXT',upper(names),'exact');
   if ~isempty(I), EXT = getfield(options,names{I}); end
   I = strmatch('U',upper(names),'exact');
   if ~isempty(I), W = getfield(options,names{I}); end
   I = strmatch('V',upper(names),'exact');
   if ~isempty(I), V = getfield(options,names{I}); end
   %%%%%%%%%%%%%%%%%%%%%%%%
end 

% Check type of input values and output an error message if needed.
if (~isnumeric(K)   || ~isnumeric(maxit) || ...
    ~isnumeric(m_b) || ~isnumeric(tol))
   error('Incorrect type for input value(s) in the structure.');
end

% Preallocate memory for W, F, and V. These matrices are full and resizing 
% will cause an increase in cpu time.
if EXT == 1
    W = zeros(m,m_b); F = zeros(n,1); 
    V = zeros(n,m_b); V(:,1) = randn(n,1);
end

% Increase the number of desired values by adjust to help increase convergence. K
% is re-adjusted as vectors converge. This is only an initial value of K.
K_org = K; K = K + adjust; 

% Check for input errors in the data structure only on FIRST call.
if EXT == 1
  if K       <= 0,   error('K must be a positive value.'),    end
  if K  >  min(n,m), error('K must be less than min(n,m) + %g.',adjust),  end   
  if m_b     <= 1,   error('M_B must be greater than 1.'),   end
  if tol     <  0,   error('TOL must be non-negative.'),      end
  if maxit   <= 0,   error('MAXIT must be positive.'),        end
  if m_b >= min(n,m)
     m_b = floor(min(n,m)-0.1);
     warning(['Changing M_B to ',num2str(m_b)]);
     if m_b - K - 1  < 0
      adjust = 0; K = m_b - 1; 
      warning(['Changing adjust to ',num2str(adjust)]);
      warning(['Changing K to ',num2str(K)]);
     end
  end
  if m_b - K - 1  < 0
    m_b = ceil(K+1+0.1); 
    warning(['Changing M_B to ',num2str(m_b)]);
  end
  if m_b >= min(n,m)
    m_b = floor(min(n,m)-0.1);
    adjust = 0; K = m_b - 1; 
    warning(['Changing adjust to ',num2str(adjust)]);
    warning(['Changing K to ',num2str(K)]);
    warning(['Changing M_B to ',num2str(m_b)]);
  end
end
 
% Set tolerance to machine precision if tol < eps.
if tol < eps, tol = eps; end

%--------------------------%
% END: PARSE INPUT VALUES. %
%--------------------------%

%-----------------------------------------------------------%
% BEGIN: DESCRIPTION AND INITIALIZATION OF LOCAL VARIABLES. %
%-----------------------------------------------------------%

% Initialization and description of local variables.   
if EXT == 1,B = []; end                                   % Bidiagonal matrix.
if EXT == 1,Bsz =[]; else Bsz = size(B,1); end            % Size of the bidiagonal matrix.
eps23 = eps^(2/3);   % Two thirds of eps, used for Smax. Avoids using zero.
I=[];                % Used for indexing.
iter = 1;            % Main loop iteration count.
J=[];                % Used for indexing.
mprod = 0;           % The number of matrix vector products.
R_F =[];             % Two norm of the residual vector F.
sqrteps = sqrt(eps); % Square root of machine tolerance used in convergence testing.
Smax = 1;            % Holds the maximum value of all computed singular values of B est. ||A||_2.
Smin=[];             % Holds the minimum value of all computed singular values of B est. cond(A).
SVTol = min(sqrteps,tol);  % Tolerance to determine when a singular vector has converged.
S_B =[];             % Singular values of B.
U_B =[];             % Left singular vectors of B.
V_B =[];             % Right singular vectors of B.


%--------------------------------------------------------------------%
% END: DESCRIPTION AND INITIALIZATION OF LOCAL AND GLOBAL VARIABLES. %
%--------------------------------------------------------------------%

%----------------------------%
% BEGIN: MAIN ITERATION LOOP %
%----------------------------%

while (iter <= maxit)
    
    % Compute the Lanczos bidiagonalization decomposition.
    [V,W,F,B,mprod] = ablanzbd(varargin{1},V,W,F,B,K,m_b,n,m,mprod,SVTol*Smax,reorth,iter);

    % Determine the size of the bidiagonal matrix B.
    Bsz = size(B,1);  

    % Compute the norm of the vector F, and normalize F.
    R_F = norm(F); F = F/R_F; 

    % Compute singular triplets of B. MATLAB's svds orders the singular values
    % largest to smallest.
    [U_B,S_B,V_B] = svd(B); S_B = diag(S_B);
    
    % Estimate ||A|| using the largest singular value over all iterations
    % and estimate the cond(A) using approximations to the largest and smallest 
    % singular values. If a small singular value is less than sqrteps use only Ritz
    % vectors to augment and require two-sided reorthogonalization.
    if iter==1
       Smax = S_B(1); Smin = S_B(Bsz);
    else 
       Smax = max(Smax,S_B(1)); Smin = min(Smin,S_B(Bsz));
    end
    Smax = max(eps23,Smax);
    if Smin/Smax < sqrteps, reorth = 'TWO'; end 
   
    % Compute the residuals for the singular values.
    R = R_F*U_B(Bsz,:);  
    
    % Check for convergence.
    Len_res = length(find(abs(R(1:K_org))' < tol*Smax));
    if Len_res == K_org 
      U_B = U_B(:,1:K_org); V_B = V_B(:,1:K_org); S_B = S_B(1:K_org);
      break;
    end
    
    % Reached maximum number of iterations, exit main loop.
    if iter >= maxit, break, end;
    
    % Compute the starting vectors and first block B(1:K,1:K+1). 
    V(:,1:K+1) = [V*V_B(:,1:K) F];    
    B = [diag(S_B(1:K)), R(1:K)'];
 
    % Compute left approximate singular vectors.
    W(:,1:K) = W*U_B(:,1:K);

    % Update the main iteration loop count.
    iter = iter + 1;

end 

%--------------------------%
% END: MAIN ITERATION LOOP %
%--------------------------%

%-----------------------%
% BEGIN: OUTPUT RESULTS %
%-----------------------%

% Test to see if maximum number of iterations have been reached.
FLAG(1) = 0; if iter >= maxit FLAG(1) = 1; end; 
varargout{1} = W*U_B(:,1:K_org); 
varargout{2} = diag(S_B(1:K_org)); 
varargout{3} = V*V_B(:,1:K_org);
FLAG(2) = mprod;
varargout{4} = FLAG;
varargout{5} = F;
varargout{6}= [diag(S_B(1:K_org)), R(1:K_org)']; 
end

%---------------------%
% END: OUTPUT RESULTS %
%---------------------%

%------------------------------------------------%
% BEGIN: LANCZOS BIDIAGONALIZATION DECOMPOSITION %
%------------------------------------------------%

function [V,W,F,B,mprod] = ablanzbd(A,V,W,F,B,K,m_b,n,m,mprod,SVTol,reorth,iter)
% Computes the Lanczos bidiagonalization decomposition
%  
%  A*V  = W*B
%  A'*W = V*B' + F*E^T
%  
% with full reorthogonalization. The matrix A can be passed as a numeric 
% matrix or as a filename. If the M x N matrix A is a filename then
% Y = Afunc(X,M,N,'transpose'). If transpose = 'F', then X is an (N x 1)
% matrix and Y = A*X. If transpose = 'T', then X is an (M x 1) matrix
% and Y = A'*X.

% James Baglama
% DATE: 3/05/04

% Initialization of main loop count J, output matrix B, singular block indicator.
J = 1;  

% Normalize starting vector.
if iter == 1
   V(:,1) = V(:,1)/norm(V(:,1)); B=[];
else
   J = K+1;
end

% Matrix A product with vector(s) V, (W=A*V).
  if ischar(A)
     W(:,J) = feval(A,V(:,J),m,n,'F');
  else
     W(:,J) = A*V(:,J);
  end

% Count the number of matrix vector products.
mprod = mprod + 1;

% Input vectors are singular vectors and AV(:,J) which must be orthogonalized.
if iter ~= 1
   W(:,J) = orthog(W(:,J),W(:,1:J-1));
end

% Compute the norm of W.
S = norm(W(:,J)); 

% Check for linearly dependent vectors.
if S <= SVTol
   W(:,J) = randn(size(W,1),1);
   W(:,J) = orthog(W(:,J),W(:,1:J-1));
   W(:,J) = W(:,J)/norm(W(:,J));
   S = 0;
else
   W(:,J) = W(:,J)/S;
end

% Begin of main iteration loop for the block Lanczos bidiagonalization decomposition.
while (J <= m_b)

   % Matrix A' product with vector(s), (F = A'*W). 

      if ischar(A)
        F = feval(A,W(:,J),m,n,'T');
      else
        F = (W(:,J)'*A)';
      end
   
   % Count the number of matrix vector products.
   mprod = mprod + 1;
   
   % One step of the block classical Gram-Schmidt process.
   F = F - V(:,J)*S';

   % Full Reorthogonalization step. "Short vectors".
   F = orthog(F,V(:,1:J));
   
   if J+1 <= m_b

      % Compute the norm of F.
      R = norm(F); 
      
      % Check for linearly dependent vectors.
      if R <= SVTol
         F = randn(size(V,1),1); 
         F = orthog(F,V(:,1:J));
         V(:,J+1) = F/norm(F);
         R = 0;
      else
        V(:,J+1) = F/R;
      end 

      % Compute block bidiagonal matrix B. 
      if isempty(B) 
         B = [S R'];
      else
         B = [B zeros(J-1,1); zeros(1,J-1) S R'];
      end 
    
      % Matrix A product with vector(s), (W=A*V).
         if ischar(A)
            W(:,J+1) = feval(A,V(:,J+1),m,n,'F');
         else
            W(:,J+1) = A*V(:,J+1);
         end
 

      % Count the number of matrix vector products.
      mprod = mprod + 1;

      % One step of the block classical Gram-Schmidt process.
      W(:,J+1) =  W(:,J+1) - W(:,J)*R';

      % Full Reorthogonalization step. "Long vectors"
      if ( iter == 1 | strcmp(reorth,'TWO') )
         W(:,J+1) = orthog(W(:,J+1),W(:,1:J));
      end
   
      % Compute the norm of W.
      S = norm(W(:,J+1)); 
      
     % Check for linearly dependent vectors.
     if S <= SVTol 
        W(:,J+1) = randn(size(W,1),1);
        W(:,J+1) = orthog(W(:,J+1),W(:,1:J));
        W(:,J+1) = W(:,J+1)/norm(W(:,J+1));
        S = 0;
     else
        W(:,J+1) = W(:,J+1)/S;
     end 

   else
    
    % Add last block to matrix B
     B = [B; zeros(1,J-1) S]; 

   end

    % Update iteration count.
    J = J + 1;

end
end

function Y = orthog(Y,X)
% Orthogonalize vectors Y against vectors X.

if size(X,2) < size(Y,2), dotY = X'*Y; else dotY = (Y'*X)'; end
Y = Y - X*dotY; 
end

 
%----------------------------------------------------%
% END: BLOCK LANCZOS BIDIAGONALIZATION DECOMPOSITION %
%----------------------------------------------------%

