%TEST_HA Test program for HUBAUTH package.
%   This script identifies the most important nodes of two test networks,
%   according to subgraph centrality, using four different methods.

%   James Baglama, University of Rhode Island, U.S.A.
%   Caterina Fenu & Giuseppe Rodriguez, University of Cagliari, Italy
%   Lothar Reichel, Kent State University, U.S.A.
%   Email: jbaglama@math.uri.edu, reichel@math.kent.edu
%          kate.fenu@gmail.com, rodriguez@unica.it, 
%
%   Last revised August 4, 2013

% test matrices
testmats = {
	'airlines'
	'celegans'
	};
nmatr = size(testmats,1);

clc
fprintf('\nComputation of hub/authority centrality.\n\n')
for i = 1:nmatr
	fprintf('%3d - %s\n', i, testmats{i})
end
fprintf('\nPress enter for default values.\n')
tst = input('choose a test matrix (def. 1): ');
if isempty(tst), tst = 1; end
load(sprintf('adjacency_%s',testmats{tst}))
n = size(A,1);
fprintf('number of nodes: %d\n',n)
numnod = input('choose the number of nodes to be identified (def. 5): ');
if isempty(numnod), numnod = 5; end

matrixname = testmats{tst};

% options
clear opts
opts.gausstolq = 1e-3;		% quadrature accuracy
opts.sgctoll = 1e-3;		% low-rank accuracy
opts.sgcshift = 1;		% spectrum shift
opts.gaussreorth = 0;		% reorthogonalization

fprintf('Computing subgraph centrality by expm ')
tic
AA = [zeros(n) A;A' zeros(n)];
allx = diag(expm(AA));
[hcentr ih] = sort(allx(1:n),'descend');
[acentr ia] = sort(allx(n+1:2*n),'descend');
hc = hcentr(1:numnod);		% HUB centr
i_h = ih(1:numnod);		% VIP nodes Hub
ac = acentr(1:numnod);		% AUTH centr
i_a = ia(1:numnod);		% VIP nodes Auth
fprintf('(%.2f seconds)\n',toc)

fprintf('Computing hub/authority centrality by Gauss-Lanczos ')
tic
[vipgh vipga gaussh gaussa infog] = hubauthgauss(A,numnod,opts,0);
fprintf('(%.2f seconds)\n',toc)

fprintf('Computing hub/authority centrality by low-rank approx. (weak conv.) ')
tic
[viplrh viplra hcenlr acenlr infolr] = hubauthlr(A,numnod,opts);
viplrh = viplrh(1:numnod);
viplra = viplra(1:numnod);
hcenlr = hcenlr(1:numnod);
acenlr = acenlr(1:numnod);
fprintf('(%.2f seconds)\n',toc)

fprintf('Computing hub/authority centrality by hybrid algorithm ')
tic
[viphh vipha hybcenh hybcena infoh] = hubauthhyb(A,numnod,opts);
fprintf('(%.2f seconds)\n',toc)

results = [ (1:numnod)' i_h hc vipgh hcenlr viplrh gaussh viphh hybcenh ];
fprintf('\n          EXPM           GAUSS          LOW-RANK         HYBRID \n')
fprintf('%3d.%5d(%.2e) %5d(%.2e) %5d(%.2e) %5d(%.2e)\n', results')
fprintf('\nThe table displays the indexes of the nodes with the largest\n')
fprintf('hub centrality, which is reported in parentheses.\n\n')

results = [ (1:numnod)' i_a ac vipga acenlr viplra gaussa vipha hybcena ];
fprintf('\n          EXPM           GAUSS          LOW-RANK         HYBRID \n')
fprintf('%3d.%5d(%.2e) %5d(%.2e) %5d(%.2e) %5d(%.2e)\n', results')
fprintf('\nThe table displays the indexes of the nodes with the largest\n')
fprintf('authority centrality, which is reported in parentheses.\n\n')

